@@ -18,17 +18,37 @@
 #' 2.1. Parameters for Tree Booster
 #'
 #' \itemize{
-#'   \item \code{eta} control the learning rate: scale the contribution of each tree by a factor of \code{0 < eta < 1} when it is added to the current approximation. Used to prevent overfitting by making the boosting process more conservative. Lower value for \code{eta} implies larger value for \code{nrounds}: low \code{eta} value means model more robust to overfitting but slower to compute. Default: 0.3
-#'   \item \code{gamma} minimum loss reduction required to make a further partition on a leaf node of the tree. the larger, the more conservative the algorithm will be.
+#'   \item{ \code{eta} control the learning rate: scale the contribution of each tree by a factor of \code{0 < eta < 1}
+#'          when it is added to the current approximation.
+#'          Used to prevent overfitting by making the boosting process more conservative.
+#'          Lower value for \code{eta} implies larger value for \code{nrounds}: low \code{eta} value means model
+#'          more robust to overfitting but slower to compute. Default: 0.3}
+#'   \item{ \code{gamma} minimum loss reduction required to make a further partition on a leaf node of the tree.
+#'          the larger, the more conservative the algorithm will be.}
 #'   \item \code{max_depth} maximum depth of a tree. Default: 6
-#'   \item \code{min_child_weight} minimum sum of instance weight (hessian) needed in a child. If the tree partition step results in a leaf node with the sum of instance weight less than min_child_weight, then the building process will give up further partitioning. In linear regression mode, this simply corresponds to minimum number of instances needed to be in each node. The larger, the more conservative the algorithm will be. Default: 1
-#'   \item \code{subsample} subsample ratio of the training instance. Setting it to 0.5 means that xgboost randomly collected half of the data instances to grow trees and this will prevent overfitting. It makes computation shorter (because less data to analyse). It is advised to use this parameter with \code{eta} and increase \code{nrounds}. Default: 1
+#'   \item{\code{min_child_weight} minimum sum of instance weight (hessian) needed in a child.
+#'         If the tree partition step results in a leaf node with the sum of instance weight less than min_child_weight,
+#'         then the building process will give up further partitioning.
+#'         In linear regression mode, this simply corresponds to minimum number of instances needed to be in each node.
+#'         The larger, the more conservative the algorithm will be. Default: 1}
+#'   \item{ \code{subsample} subsample ratio of the training instance.
+#'         Setting it to 0.5 means that xgboost randomly collected half of the data instances to grow trees
+#'         and this will prevent overfitting. It makes computation shorter (because less data to analyse).
+#'         It is advised to use this parameter with \code{eta} and increase \code{nrounds}. Default: 1}
 #'   \item \code{colsample_bytree} subsample ratio of columns when constructing each tree. Default: 1
 #'   \item \code{lambda} L2 regularization term on weights. Default: 1
 #'   \item \code{alpha} L1 regularization term on weights. (there is no L1 reg on bias because it is not important). Default: 0
-#'   \item \code{num_parallel_tree} Experimental parameter. number of trees to grow per round. Useful to test Random Forest through XGBoost (set \code{colsample_bytree < 1}, \code{subsample  < 1}  and \code{round = 1}) accordingly. Default: 1
-#'   \item \code{monotone_constraints} A numerical vector consists of \code{1}, \code{0} and \code{-1} with its length equals to the number of features in the training data. \code{1} is increasing, \code{-1} is decreasing and \code{0} is no constraint.
-#'   \item \code{interaction_constraints} A list of vectors specifying feature indices of permitted interactions. Each item of the list represents one permitted interaction where specified features are allowed to interact with each other. Feature index values should start from \code{0} (\code{0} references the first column).  Leave argument unspecified for no interaction constraints.
+#'   \item{ \code{num_parallel_tree} Experimental parameter. number of trees to grow per round.
+#'          Useful to test Random Forest through XGBoost
+#'          (set \code{colsample_bytree < 1}, \code{subsample  < 1}  and \code{round = 1}) accordingly.
+#'          Default: 1}
+#'   \item{ \code{monotone_constraints} A numerical vector consists of \code{1}, \code{0} and \code{-1} with its length
+#'          equals to the number of features in the training data.
+#'          \code{1} is increasing, \code{-1} is decreasing and \code{0} is no constraint.}
+#'   \item{ \code{interaction_constraints} A list of vectors specifying feature indices of permitted interactions.
+#'        Each item of the list represents one permitted interaction where specified features are allowed to interact with each other.
+#'        Feature index values should start from \code{0} (\code{0} references the first column).
+#'        Leave argument unspecified for no interaction constraints.}
 #' }
 #'
 #' 2.2. Parameters for Linear Booster
@@ -42,29 +62,53 @@
 #' 3. Task Parameters
 #'
 #' \itemize{
-#' \item \code{objective} specify the learning task and the corresponding learning objective, users can pass a self-defined function to it. The default objective options are below:
+#' \item{ \code{objective} specify the learning task and the corresponding learning objective, users can pass a self-defined function to it.
+#'        The default objective options are below:
 #'   \itemize{
 #'     \item \code{reg:squarederror} Regression with squared loss (Default).
-#'     \item \code{reg:squaredlogerror}: regression with squared log loss \eqn{1/2 * (log(pred + 1) - log(label + 1))^2}. All inputs are required to be greater than -1. Also, see metric rmsle for possible issue with this objective.
+#'     \item{ \code{reg:squaredlogerror}: regression with squared log loss \eqn{1/2 * (log(pred + 1) - log(label + 1))^2}.
+#'            All inputs are required to be greater than -1.
+#'            Also, see metric rmsle for possible issue with this objective.}
 #'     \item \code{reg:logistic} logistic regression.
 #'     \item \code{reg:pseudohubererror}: regression with Pseudo Huber loss, a twice differentiable alternative to absolute loss.
 #'     \item \code{binary:logistic} logistic regression for binary classification. Output probability.
 #'     \item \code{binary:logitraw} logistic regression for binary classification, output score before logistic transformation.
 #'     \item \code{binary:hinge}: hinge loss for binary classification. This makes predictions of 0 or 1, rather than producing probabilities.
-#'     \item \code{count:poisson}: Poisson regression for count data, output mean of Poisson distribution. \code{max_delta_step} is set to 0.7 by default in poisson regression (used to safeguard optimization).
-#'     \item \code{survival:cox}: Cox regression for right censored survival time data (negative values are considered right censored). Note that predictions are returned on the hazard ratio scale (i.e., as HR = exp(marginal_prediction) in the proportional hazard function \code{h(t) = h0(t) * HR)}.
-#'     \item \code{survival:aft}: Accelerated failure time model for censored survival time data. See \href{https://xgboost.readthedocs.io/en/latest/tutorials/aft_survival_analysis.html}{Survival Analysis with Accelerated Failure Time} for details.
+#'     \item{ \code{count:poisson}: Poisson regression for count data, output mean of Poisson distribution.
+#'            \code{max_delta_step} is set to 0.7 by default in poisson regression (used to safeguard optimization).}
+#'     \item{ \code{survival:cox}: Cox regression for right censored survival time data (negative values are considered right censored).
+#'            Note that predictions are returned on the hazard ratio scale (i.e., as HR = exp(marginal_prediction) in the proportional
+#'            hazard function \code{h(t) = h0(t) * HR)}.}
+#'     \item{ \code{survival:aft}: Accelerated failure time model for censored survival time data. See
+#'            \href{https://xgboost.readthedocs.io/en/latest/tutorials/aft_survival_analysis.html}{Survival Analysis with Accelerated Failure Time}
+#'            for details.}
 #'     \item \code{aft_loss_distribution}: Probability Density Function used by \code{survival:aft} and \code{aft-nloglik} metric.
-#'     \item \code{multi:softmax} set xgboost to do multiclass classification using the softmax objective. Class is represented by a number and should be from 0 to \code{num_class - 1}.
-#'     \item \code{multi:softprob} same as softmax, but prediction outputs a vector of ndata * nclass elements, which can be further reshaped to ndata, nclass matrix. The result contains predicted probabilities of each data point belonging to each class.
+#'     \item{ \code{multi:softmax} set xgboost to do multiclass classification using the softmax objective.
+#'            Class is represented by a number and should be from 0 to \code{num_class - 1}.}
+#'     \item{ \code{multi:softprob} same as softmax, but prediction outputs a vector of ndata * nclass elements, which can be
+#'            further reshaped to ndata, nclass matrix. The result contains predicted probabilities of each data point belonging
+#'            to each class.}
 #'     \item \code{rank:pairwise} set xgboost to do ranking task by minimizing the pairwise loss.
-#'     \item \code{rank:ndcg}: Use LambdaMART to perform list-wise ranking where \href{https://en.wikipedia.org/wiki/Discounted_cumulative_gain}{Normalized Discounted Cumulative Gain (NDCG)} is maximized.
-#'     \item \code{rank:map}: Use LambdaMART to perform list-wise ranking where \href{https://en.wikipedia.org/wiki/Evaluation_measures_(information_retrieval)#Mean_average_precision}{Mean Average Precision (MAP)} is maximized.
-#'     \item \code{reg:gamma}: gamma regression with log-link. Output is a mean of gamma distribution. It might be useful, e.g., for modeling insurance claims severity, or for any outcome that might be \href{https://en.wikipedia.org/wiki/Gamma_distribution#Applications}{gamma-distributed}.
-#'     \item \code{reg:tweedie}: Tweedie regression with log-link. It might be useful, e.g., for modeling total loss in insurance, or for any outcome that might be \href{https://en.wikipedia.org/wiki/Tweedie_distribution#Applications}{Tweedie-distributed}.
+#'     \item{ \code{rank:ndcg}: Use LambdaMART to perform list-wise ranking where
+#'            \href{https://en.wikipedia.org/wiki/Discounted_cumulative_gain}{Normalized Discounted Cumulative Gain (NDCG)} is maximized.}
+#'     \item{ \code{rank:map}: Use LambdaMART to perform list-wise ranking where
+#'            \href{https://en.wikipedia.org/wiki/Evaluation_measures_(information_retrieval)#Mean_average_precision}{Mean Average Precision (MAP)}
+#'            is maximized.}
+#'     \item{ \code{reg:gamma}: gamma regression with log-link.
+#'            Output is a mean of gamma distribution.
+#'            It might be useful, e.g., for modeling insurance claims severity, or for any outcome that might be
+#'            \href{https://en.wikipedia.org/wiki/Gamma_distribution#Applications}{gamma-distributed}.}
+#'     \item{ \code{reg:tweedie}: Tweedie regression with log-link.
+#'            It might be useful, e.g., for modeling total loss in insurance, or for any outcome that might be
+#'            \href{https://en.wikipedia.org/wiki/Tweedie_distribution#Applications}{Tweedie-distributed}.}
 #'   }
+#'  }
 #'   \item \code{base_score} the initial prediction score of all instances, global bias. Default: 0.5
-#'   \item \code{eval_metric} evaluation metrics for validation data. Users can pass a self-defined function to it. Default: metric will be assigned according to objective(rmse for regression, and error for classification, mean average precision for ranking). List is provided in detail section.
+#'   \item{ \code{eval_metric} evaluation metrics for validation data.
+#'          Users can pass a self-defined function to it.
+#'          Default: metric will be assigned according to objective
+#'          (rmse for regression, and error for classification, mean average precision for ranking).
+#'          List is provided in detail section.}
 #' }
 #'
 #' @param data training dataset. \code{xgb.train} accepts only an \code{xgb.DMatrix} as the input.
@@ -141,7 +185,8 @@
 #'      \item \code{merror} Multiclass classification error rate. It is calculated as \code{(# wrong cases) / (# all cases)}.
 #'      \item \code{mae} Mean absolute error
 #'      \item \code{mape} Mean absolute percentage error
-#'      \item \code{auc} Area under the curve. \url{https://en.wikipedia.org/wiki/Receiver_operating_characteristic#'Area_under_curve} for ranking evaluation.
+#'      \item{ \code{auc} Area under the curve.
+#'             \url{https://en.wikipedia.org/wiki/Receiver_operating_characteristic#'Area_under_curve} for ranking evaluation.}
 #'      \item \code{aucpr} Area under the PR curve. \url{https://en.wikipedia.org/wiki/Precision_and_recall} for ranking evaluation.
 #'      \item \code{ndcg} Normalized Discounted Cumulative Gain (for ranking task). \url{https://en.wikipedia.org/wiki/NDCG}
 #'   }
@@ -276,6 +321,10 @@ xgb.train <- function(params = list(), data, nrounds, watchlist = list(),
     if (is.null(evnames) || any(evnames == ""))
       stop("each element of the watchlist must have a name tag")
   }
+  # Handle multiple evaluation metrics given as a list
+  for (m in params$eval_metric) {
+    params <- c(params, list(eval_metric = m))
+  }
 
   # evaluation printing callback
   params <- c(params)
@@ -314,8 +363,13 @@ xgb.train <- function(params = list(), data, nrounds, watchlist = list(),
   is_update <- NVL(params[['process_type']], '.') == 'update'
 
   # Construct a booster (either a new one or load from xgb_model)
-  handle <- xgb.Booster.handle(params, append(watchlist, dtrain), xgb_model)
-  bst <- xgb.handleToBooster(handle)
+  handle <- xgb.Booster.handle(
+    params = params,
+    cachelist = append(watchlist, dtrain),
+    modelfile = xgb_model,
+    handle = NULL
+  )
+  bst <- xgb.handleToBooster(handle = handle, raw = NULL)
 
   # extract parameters that can affect the relationship b/w #trees and #iterations
   num_class <- max(as.numeric(NVL(params[['num_class']], 1)), 1)
@@ -341,10 +395,21 @@ xgb.train <- function(params = list(), data, nrounds, watchlist = list(),
 
     for (f in cb$pre_iter) f()
 
-    xgb.iter.update(bst$handle, dtrain, iteration - 1, obj)
+    xgb.iter.update(
+        booster_handle = bst$handle,
+        dtrain = dtrain,
+        iter = iteration - 1,
+        obj = obj
+    )
 
-    if (length(watchlist) > 0)
-      bst_evaluation <- xgb.iter.eval(bst$handle, watchlist, iteration - 1, feval)
+    if (length(watchlist) > 0) {
+      bst_evaluation <- xgb.iter.eval(  # nolint: object_usage_linter
+        booster_handle = bst$handle,
+        watchlist = watchlist,
+        iter = iteration - 1,
+        feval = feval
+      )
+    }
 
     xgb.attr(bst$handle, 'niter') <- iteration - 1
 
