@@ -1,11 +1,11 @@
 /**
- * Copyright 2014-2022 by XGBoost Contributors
+ * Copyright 2014-2023 by XGBoost Contributors
  */
 #include <dmlc/common.h>
 #include <dmlc/omp.h>
 #include <xgboost/c_api.h>
+#include <xgboost/context.h>
 #include <xgboost/data.h>
-#include <xgboost/generic_parameters.h>
 #include <xgboost/logging.h>
 
 #include <cstdio>
@@ -16,9 +16,11 @@
 #include <vector>
 
 #include "../../src/c_api/c_api_error.h"
+#include "../../src/c_api/c_api_utils.h"  // MakeSparseFromPtr
 #include "../../src/common/threading_utils.h"
 
-#include "./xgboost_R.h"
+#include "./xgboost_R.h"  // Must follow other includes.
+#include "Rinternals.h"
 
 /*!
  * \brief macro to annotate begin of api
@@ -46,14 +48,14 @@
 
 using dmlc::BeginPtr;
 
-xgboost::GenericParameter const *BoosterCtx(BoosterHandle handle) {
+xgboost::Context const *BoosterCtx(BoosterHandle handle) {
   CHECK_HANDLE();
   auto *learner = static_cast<xgboost::Learner *>(handle);
   CHECK(learner);
   return learner->Ctx();
 }
 
-xgboost::GenericParameter const *DMatrixCtx(DMatrixHandle handle) {
+xgboost::Context const *DMatrixCtx(DMatrixHandle handle) {
   CHECK_HANDLE();
   auto p_m = static_cast<std::shared_ptr<xgboost::DMatrix> *>(handle);
   CHECK(p_m);
@@ -114,13 +116,29 @@ XGB_DLL SEXP XGDMatrixCreateFromMat_R(SEXP mat, SEXP missing, SEXP n_threads) {
     din = REAL(mat);
   }
   std::vector<float> data(nrow * ncol);
-  int32_t threads = xgboost::common::OmpGetNumThreads(asInteger(n_threads));
+  xgboost::Context ctx;
+  ctx.nthread = asInteger(n_threads);
+  std::int32_t threads = ctx.Threads();
+
+  if (is_int) {
+    xgboost::common::ParallelFor(nrow, threads, [&](xgboost::omp_ulong i) {
+      for (size_t j = 0; j < ncol; ++j) {
+        auto v = iin[i + nrow * j];
+        if (v == NA_INTEGER) {
+          data[i * ncol + j] = std::numeric_limits<float>::quiet_NaN();
+        } else {
+          data[i * ncol + j] = static_cast<float>(v);
+        }
+      }
+    });
+  } else {
+    xgboost::common::ParallelFor(nrow, threads, [&](xgboost::omp_ulong i) {
+      for (size_t j = 0; j < ncol; ++j) {
+        data[i * ncol + j] = din[i + nrow * j];
+      }
+    });
+  }
 
-  xgboost::common::ParallelFor(nrow, threads, [&](xgboost::omp_ulong i) {
-    for (size_t j = 0; j < ncol; ++j) {
-      data[i * ncol + j] = is_int ? static_cast<float>(iin[i + nrow * j]) : din[i + nrow * j];
-    }
-  });
   DMatrixHandle handle;
   CHECK_CALL(XGDMatrixCreateFromMat_omp(BeginPtr(data), nrow, ncol,
                                         asReal(missing), &handle, threads));
@@ -131,66 +149,78 @@ XGB_DLL SEXP XGDMatrixCreateFromMat_R(SEXP mat, SEXP missing, SEXP n_threads) {
   return ret;
 }
 
-XGB_DLL SEXP XGDMatrixCreateFromCSC_R(SEXP indptr, SEXP indices, SEXP data,
-                                      SEXP num_row, SEXP n_threads) {
-  SEXP ret;
-  R_API_BEGIN();
+namespace {
+void CreateFromSparse(SEXP indptr, SEXP indices, SEXP data, std::string *indptr_str,
+                      std::string *indices_str, std::string *data_str) {
   const int *p_indptr = INTEGER(indptr);
   const int *p_indices = INTEGER(indices);
   const double *p_data = REAL(data);
-  size_t nindptr = static_cast<size_t>(length(indptr));
-  size_t ndata = static_cast<size_t>(length(data));
-  size_t nrow = static_cast<size_t>(INTEGER(num_row)[0]);
-  std::vector<size_t> col_ptr_(nindptr);
-  std::vector<unsigned> indices_(ndata);
-  std::vector<float> data_(ndata);
 
-  for (size_t i = 0; i < nindptr; ++i) {
-    col_ptr_[i] = static_cast<size_t>(p_indptr[i]);
-  }
-  int32_t threads = xgboost::common::OmpGetNumThreads(asInteger(n_threads));
-  xgboost::common::ParallelFor(ndata, threads, [&](xgboost::omp_ulong i) {
-    indices_[i] = static_cast<unsigned>(p_indices[i]);
-    data_[i] = static_cast<float>(p_data[i]);
-  });
+  auto nindptr = static_cast<std::size_t>(length(indptr));
+  auto ndata = static_cast<std::size_t>(length(data));
+  CHECK_EQ(ndata, p_indptr[nindptr - 1]);
+  xgboost::detail::MakeSparseFromPtr(p_indptr, p_indices, p_data, nindptr, indptr_str, indices_str,
+                                     data_str);
+}
+}  // namespace
+
+XGB_DLL SEXP XGDMatrixCreateFromCSC_R(SEXP indptr, SEXP indices, SEXP data, SEXP num_row,
+                                      SEXP missing, SEXP n_threads) {
+  SEXP ret;
+  R_API_BEGIN();
+  std::int32_t threads = asInteger(n_threads);
+
+  using xgboost::Integer;
+  using xgboost::Json;
+  using xgboost::Object;
+
+  std::string sindptr, sindices, sdata;
+  CreateFromSparse(indptr, indices, data, &sindptr, &sindices, &sdata);
+  auto nrow = static_cast<std::size_t>(INTEGER(num_row)[0]);
+
   DMatrixHandle handle;
-  CHECK_CALL(XGDMatrixCreateFromCSCEx(BeginPtr(col_ptr_), BeginPtr(indices_),
-                                      BeginPtr(data_), nindptr, ndata,
-                                      nrow, &handle));
+  Json jconfig{Object{}};
+  // Construct configuration
+  jconfig["nthread"] = Integer{threads};
+  jconfig["missing"] = xgboost::Number{asReal(missing)};
+  std::string config;
+  Json::Dump(jconfig, &config);
+  CHECK_CALL(XGDMatrixCreateFromCSC(sindptr.c_str(), sindices.c_str(), sdata.c_str(), nrow,
+                                    config.c_str(), &handle));
+
   ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+
   R_RegisterCFinalizerEx(ret, _DMatrixFinalizer, TRUE);
   R_API_END();
   UNPROTECT(1);
   return ret;
 }
 
-XGB_DLL SEXP XGDMatrixCreateFromCSR_R(SEXP indptr, SEXP indices, SEXP data,
-                                      SEXP num_col, SEXP n_threads) {
+XGB_DLL SEXP XGDMatrixCreateFromCSR_R(SEXP indptr, SEXP indices, SEXP data, SEXP num_col,
+                                      SEXP missing, SEXP n_threads) {
   SEXP ret;
   R_API_BEGIN();
-  const int *p_indptr = INTEGER(indptr);
-  const int *p_indices = INTEGER(indices);
-  const double *p_data = REAL(data);
-  size_t nindptr = static_cast<size_t>(length(indptr));
-  size_t ndata = static_cast<size_t>(length(data));
-  size_t ncol = static_cast<size_t>(INTEGER(num_col)[0]);
-  std::vector<size_t> row_ptr_(nindptr);
-  std::vector<unsigned> indices_(ndata);
-  std::vector<float> data_(ndata);
+  std::int32_t threads = asInteger(n_threads);
+
+  using xgboost::Integer;
+  using xgboost::Json;
+  using xgboost::Object;
+
+  std::string sindptr, sindices, sdata;
+  CreateFromSparse(indptr, indices, data, &sindptr, &sindices, &sdata);
+  auto ncol = static_cast<std::size_t>(INTEGER(num_col)[0]);
 
-  for (size_t i = 0; i < nindptr; ++i) {
-    row_ptr_[i] = static_cast<size_t>(p_indptr[i]);
-  }
-  int32_t threads = xgboost::common::OmpGetNumThreads(asInteger(n_threads));
-  xgboost::common::ParallelFor(ndata, threads, [&](xgboost::omp_ulong i) {
-    indices_[i] = static_cast<unsigned>(p_indices[i]);
-    data_[i] = static_cast<float>(p_data[i]);
-  });
   DMatrixHandle handle;
-  CHECK_CALL(XGDMatrixCreateFromCSREx(BeginPtr(row_ptr_), BeginPtr(indices_),
-                                      BeginPtr(data_), nindptr, ndata,
-                                      ncol, &handle));
+  Json jconfig{Object{}};
+  // Construct configuration
+  jconfig["nthread"] = Integer{threads};
+  jconfig["missing"] = xgboost::Number{asReal(missing)};
+  std::string config;
+  Json::Dump(jconfig, &config);
+  CHECK_CALL(XGDMatrixCreateFromCSR(sindptr.c_str(), sindices.c_str(), sdata.c_str(), ncol,
+                                    config.c_str(), &handle));
   ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+
   R_RegisterCFinalizerEx(ret, _DMatrixFinalizer, TRUE);
   R_API_END();
   UNPROTECT(1);
@@ -422,27 +452,6 @@ XGB_DLL SEXP XGBoosterEvalOneIter_R(SEXP handle, SEXP iter, SEXP dmats, SEXP evn
   return mkString(ret);
 }
 
-XGB_DLL SEXP XGBoosterPredict_R(SEXP handle, SEXP dmat, SEXP option_mask,
-                                SEXP ntree_limit, SEXP training) {
-  SEXP ret;
-  R_API_BEGIN();
-  bst_ulong olen;
-  const float *res;
-  CHECK_CALL(XGBoosterPredict(R_ExternalPtrAddr(handle),
-                              R_ExternalPtrAddr(dmat),
-                              asInteger(option_mask),
-                              asInteger(ntree_limit),
-                              asInteger(training),
-                              &olen, &res));
-  ret = PROTECT(allocVector(REALSXP, olen));
-  for (size_t i = 0; i < olen; ++i) {
-    REAL(ret)[i] = res[i];
-  }
-  R_API_END();
-  UNPROTECT(1);
-  return ret;
-}
-
 XGB_DLL SEXP XGBoosterPredictFromDMatrix_R(SEXP handle, SEXP dmat, SEXP json_config)  {
   SEXP r_out_shape;
   SEXP r_out_result;
