@@ -59,11 +59,12 @@ XGB_DLL SEXP XGDMatrixCreateFromMat_R(SEXP mat,
  * \param indices row indices
  * \param data content of the data
  * \param num_row numer of rows (when it's set to 0, then guess from data)
+ * \param missing which value to represent missing value
  * \param n_threads Number of threads used to construct DMatrix from csc matrix.
  * \return created dmatrix
  */
 XGB_DLL SEXP XGDMatrixCreateFromCSC_R(SEXP indptr, SEXP indices, SEXP data, SEXP num_row,
-                                      SEXP n_threads);
+                                      SEXP missing, SEXP n_threads);
 
 /*!
  * \brief create a matrix content from CSR format
@@ -71,11 +72,12 @@ XGB_DLL SEXP XGDMatrixCreateFromCSC_R(SEXP indptr, SEXP indices, SEXP data, SEXP
  * \param indices column indices
  * \param data content of the data
  * \param num_col numer of columns (when it's set to 0, then guess from data)
+ * \param missing which value to represent missing value
  * \param n_threads Number of threads used to construct DMatrix from csr matrix.
  * \return created dmatrix
  */
 XGB_DLL SEXP XGDMatrixCreateFromCSR_R(SEXP indptr, SEXP indices, SEXP data, SEXP num_col,
-                                      SEXP n_threads);
+                                      SEXP missing, SEXP n_threads);
 
 /*!
  * \brief create a new dmatrix from sliced content of existing matrix
@@ -176,17 +178,6 @@ XGB_DLL SEXP XGBoosterBoostOneIter_R(SEXP handle, SEXP dtrain, SEXP grad, SEXP h
  */
 XGB_DLL SEXP XGBoosterEvalOneIter_R(SEXP handle, SEXP iter, SEXP dmats, SEXP evnames);
 
-/*!
- * \brief (Deprecated) make prediction based on dmat
- * \param handle handle
- * \param dmat data matrix
- * \param option_mask output_margin:1 predict_leaf:2
- * \param ntree_limit limit number of trees used in prediction
- * \param training Whether the prediction value is used for training.
- */
-XGB_DLL SEXP XGBoosterPredict_R(SEXP handle, SEXP dmat, SEXP option_mask,
-                                SEXP ntree_limit, SEXP training);
-
 /*!
  * \brief Run prediction on DMatrix, replacing `XGBoosterPredict_R`
  * \param handle handle
