@@ -1,5 +1,3 @@
-require(xgboost)
-
 context("Garbage Collection Safety Check")
 
 test_that("train and prediction when gctorture is on", {
@@ -9,7 +7,8 @@ test_that("train and prediction when gctorture is on", {
   test <- agaricus.test
   gctorture(TRUE)
   bst <- xgboost(data = train$data, label = train$label, max.depth = 2,
-  eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
+                 eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
   pred <- predict(bst, test$data)
   gctorture(FALSE)
+  expect_length(pred, length(test$label))
 })
