@@ -1,10 +1,11 @@
-library(testthat)
 context('Test helper functions')
 
-require(xgboost)
-require(data.table)
-require(Matrix)
-require(vcd, quietly = TRUE)
+VCD_AVAILABLE <- requireNamespace("vcd", quietly = TRUE)
+.skip_if_vcd_not_available <- function() {
+    if (!VCD_AVAILABLE) {
+        testthat::skip("Optional testing dependency 'vcd' not found.")
+    }
+}
 
 float_tolerance <- 5e-6
 
@@ -12,25 +13,30 @@ float_tolerance <- 5e-6
 flag_32bit <- .Machine$sizeof.pointer != 8
 
 set.seed(1982)
-data(Arthritis)
-df <- data.table(Arthritis, keep.rownames = FALSE)
-df[, AgeDiscret := as.factor(round(Age / 10, 0))]
-df[, AgeCat := as.factor(ifelse(Age > 30, "Old", "Young"))]
-df[, ID := NULL]
-sparse_matrix <- sparse.model.matrix(Improved~.-1, data = df) # nolint
-label <- df[, ifelse(Improved == "Marked", 1, 0)]
 
-# binary
 nrounds <- 12
-bst.Tree <- xgboost(data = sparse_matrix, label = label, max_depth = 9,
-                    eta = 1, nthread = 2, nrounds = nrounds, verbose = 0,
-                    objective = "binary:logistic", booster = "gbtree")
+if (isTRUE(VCD_AVAILABLE)) {
+    data(Arthritis, package = "vcd")
+    df <- data.table::data.table(Arthritis, keep.rownames = FALSE)
+    df[, AgeDiscret := as.factor(round(Age / 10, 0))]
+    df[, AgeCat := as.factor(ifelse(Age > 30, "Old", "Young"))]
+    df[, ID := NULL]
+    sparse_matrix <- Matrix::sparse.model.matrix(Improved~.-1, data = df) # nolint
+    label <- df[, ifelse(Improved == "Marked", 1, 0)]
 
-bst.GLM <- xgboost(data = sparse_matrix, label = label,
-                   eta = 1, nthread = 1, nrounds = nrounds, verbose = 0,
-                   objective = "binary:logistic", booster = "gblinear")
+    # binary
+    bst.Tree <- xgboost(data = sparse_matrix, label = label, max_depth = 9,
+                        eta = 1, nthread = 2, nrounds = nrounds, verbose = 0,
+                        objective = "binary:logistic", booster = "gbtree",
+                        base_score = 0.5)
 
-feature.names <- colnames(sparse_matrix)
+    bst.GLM <- xgboost(data = sparse_matrix, label = label,
+                       eta = 1, nthread = 1, nrounds = nrounds, verbose = 0,
+                       objective = "binary:logistic", booster = "gblinear",
+                       base_score = 0.5)
+
+    feature.names <- colnames(sparse_matrix)
+}
 
 # multiclass
 mlabel <- as.numeric(iris$Species) - 1
@@ -45,6 +51,7 @@ mbst.GLM <- xgboost(data = as.matrix(iris[, -5]), label = mlabel, verbose = 0,
 
 
 test_that("xgb.dump works", {
+  .skip_if_vcd_not_available()
   if (!flag_32bit)
     expect_length(xgb.dump(bst.Tree), 200)
   dump_file <- file.path(tempdir(), 'xgb.model.dump')
@@ -56,10 +63,11 @@ test_that("xgb.dump works", {
   dmp <- xgb.dump(bst.Tree, dump_format = "json")
   expect_length(dmp, 1)
   if (!flag_32bit)
-    expect_length(grep('nodeid', strsplit(dmp, '\n')[[1]]), 188)
+    expect_length(grep('nodeid', strsplit(dmp, '\n', fixed = TRUE)[[1]], fixed = TRUE), 188)
 })
 
 test_that("xgb.dump works for gblinear", {
+  .skip_if_vcd_not_available()
   expect_length(xgb.dump(bst.GLM), 14)
   # also make sure that it works properly for a sparse model where some coefficients
   # are 0 from setting large L1 regularization:
@@ -72,10 +80,11 @@ test_that("xgb.dump works for gblinear", {
   # JSON format
   dmp <- xgb.dump(bst.GLM.sp, dump_format = "json")
   expect_length(dmp, 1)
-  expect_length(grep('\\d', strsplit(dmp, '\n')[[1]]), 11)
+  expect_length(grep('\\d', strsplit(dmp, '\n', fixed = TRUE)[[1]]), 11)
 })
 
 test_that("predict leafs works", {
+  .skip_if_vcd_not_available()
   # no error for gbtree
   expect_error(pred_leaf <- predict(bst.Tree, sparse_matrix, predleaf = TRUE), regexp = NA)
   expect_equal(dim(pred_leaf), c(nrow(sparse_matrix), nrounds))
@@ -84,6 +93,7 @@ test_that("predict leafs works", {
 })
 
 test_that("predict feature contributions works", {
+  .skip_if_vcd_not_available()
   # gbtree binary classifier
   expect_error(pred_contr <- predict(bst.Tree, sparse_matrix, predcontrib = TRUE), regexp = NA)
   expect_equal(dim(pred_contr), c(nrow(sparse_matrix), ncol(sparse_matrix) + 1))
@@ -170,15 +180,16 @@ test_that("SHAPs sum to predictions, with or without DART", {
       label = y,
       nrounds = nrounds)
 
-    pr <- function(...)
+    pr <- function(...) {
       predict(fit, newdata = d, ...)
+    }
     pred <- pr()
     shap <- pr(predcontrib = TRUE)
     shapi <- pr(predinteraction = TRUE)
     tol <- 1e-5
 
     expect_equal(rowSums(shap), pred, tol = tol)
-    expect_equal(apply(shapi, 1, sum), pred, tol = tol)
+    expect_equal(rowSums(shapi), pred, tol = tol)
     for (i in seq_len(nrow(d)))
       for (f in list(rowSums, colSums))
         expect_equal(f(shapi[i, , ]), shap[i, ], tol = tol)
@@ -186,6 +197,7 @@ test_that("SHAPs sum to predictions, with or without DART", {
 })
 
 test_that("xgb-attribute functionality", {
+  .skip_if_vcd_not_available()
   val <- "my attribute value"
   list.val <- list(my_attr = val, a = 123, b = 'ok')
   list.ch <- list.val[order(names(list.val))]
@@ -219,10 +231,11 @@ test_that("xgb-attribute functionality", {
   expect_null(xgb.attributes(bst))
 })
 
-if (grepl('Windows', Sys.info()[['sysname']]) ||
-    grepl('Linux', Sys.info()[['sysname']]) ||
-    grepl('Darwin', Sys.info()[['sysname']])) {
+if (grepl('Windows', Sys.info()[['sysname']], fixed = TRUE) ||
+    grepl('Linux', Sys.info()[['sysname']], fixed = TRUE) ||
+    grepl('Darwin', Sys.info()[['sysname']], fixed = TRUE)) {
     test_that("xgb-attribute numeric precision", {
+      .skip_if_vcd_not_available()
       # check that lossless conversion works with 17 digits
       # numeric -> character -> numeric
       X <- 10^runif(100, -20, 20)
@@ -241,6 +254,7 @@ if (grepl('Windows', Sys.info()[['sysname']]) ||
 }
 
 test_that("xgb.Booster serializing as R object works", {
+  .skip_if_vcd_not_available()
   saveRDS(bst.Tree, 'xgb.model.rds')
   bst <- readRDS('xgb.model.rds')
   dtrain <- xgb.DMatrix(sparse_matrix, label = label)
@@ -259,6 +273,7 @@ test_that("xgb.Booster serializing as R object works", {
 })
 
 test_that("xgb.model.dt.tree works with and without feature names", {
+  .skip_if_vcd_not_available()
   names.dt.trees <- c("Tree", "Node", "ID", "Feature", "Split", "Yes", "No", "Missing", "Quality", "Cover")
   dt.tree <- xgb.model.dt.tree(feature_names = feature.names, model = bst.Tree)
   expect_equal(names.dt.trees, names(dt.tree))
@@ -278,16 +293,18 @@ test_that("xgb.model.dt.tree works with and without feature names", {
 
   # using integer node ID instead of character
   dt.tree.int <- xgb.model.dt.tree(model = bst.Tree, use_int_id = TRUE)
-  expect_equal(as.integer(tstrsplit(dt.tree$Yes, '-')[[2]]), dt.tree.int$Yes)
-  expect_equal(as.integer(tstrsplit(dt.tree$No, '-')[[2]]), dt.tree.int$No)
-  expect_equal(as.integer(tstrsplit(dt.tree$Missing, '-')[[2]]), dt.tree.int$Missing)
+  expect_equal(as.integer(data.table::tstrsplit(dt.tree$Yes, '-', fixed = TRUE)[[2]]), dt.tree.int$Yes)
+  expect_equal(as.integer(data.table::tstrsplit(dt.tree$No, '-', fixed = TRUE)[[2]]), dt.tree.int$No)
+  expect_equal(as.integer(data.table::tstrsplit(dt.tree$Missing, '-', fixed = TRUE)[[2]]), dt.tree.int$Missing)
 })
 
 test_that("xgb.model.dt.tree throws error for gblinear", {
+  .skip_if_vcd_not_available()
   expect_error(xgb.model.dt.tree(model = bst.GLM))
 })
 
 test_that("xgb.importance works with and without feature names", {
+  .skip_if_vcd_not_available()
   importance.Tree <- xgb.importance(feature_names = feature.names, model = bst.Tree)
   if (!flag_32bit)
     expect_equal(dim(importance.Tree), c(7, 4))
@@ -345,7 +362,8 @@ test_that("xgb.importance works with and without feature names", {
   m <- xgboost::xgboost(
     data = as.matrix(data.frame(x = c(0, 1))),
     label = c(1, 2),
-    nrounds = 1
+    nrounds = 1,
+    base_score = 0.5
   )
   df <- xgb.model.dt.tree(model = m)
   expect_equal(df$Feature, "Leaf")
@@ -353,6 +371,7 @@ test_that("xgb.importance works with and without feature names", {
 })
 
 test_that("xgb.importance works with GLM model", {
+  .skip_if_vcd_not_available()
   importance.GLM <- xgb.importance(feature_names = feature.names, model = bst.GLM)
   expect_equal(dim(importance.GLM), c(10, 2))
   expect_equal(colnames(importance.GLM), c("Feature", "Weight"))
@@ -368,6 +387,7 @@ test_that("xgb.importance works with GLM model", {
 })
 
 test_that("xgb.model.dt.tree and xgb.importance work with a single split model", {
+  .skip_if_vcd_not_available()
   bst1 <- xgboost(data = sparse_matrix, label = label, max_depth = 1,
                   eta = 1, nthread = 2, nrounds = 1, verbose = 0,
                   objective = "binary:logistic")
@@ -379,16 +399,19 @@ test_that("xgb.model.dt.tree and xgb.importance work with a single split model",
 })
 
 test_that("xgb.plot.tree works with and without feature names", {
+  .skip_if_vcd_not_available()
   expect_silent(xgb.plot.tree(feature_names = feature.names, model = bst.Tree))
   expect_silent(xgb.plot.tree(model = bst.Tree))
 })
 
 test_that("xgb.plot.multi.trees works with and without feature names", {
+  .skip_if_vcd_not_available()
   xgb.plot.multi.trees(model = bst.Tree, feature_names = feature.names, features_keep = 3)
   xgb.plot.multi.trees(model = bst.Tree, features_keep = 3)
 })
 
 test_that("xgb.plot.deepness works", {
+  .skip_if_vcd_not_available()
   d2p <- xgb.plot.deepness(model = bst.Tree)
   expect_equal(colnames(d2p), c("ID", "Tree", "Depth", "Cover", "Weight"))
   xgb.plot.deepness(model = bst.Tree, which = "med.depth")
@@ -396,6 +419,7 @@ test_that("xgb.plot.deepness works", {
 })
 
 test_that("xgb.shap.data works when top_n is provided", {
+  .skip_if_vcd_not_available()
   data_list <- xgb.shap.data(data = sparse_matrix, model = bst.Tree, top_n = 2)
   expect_equal(names(data_list), c("data", "shap_contrib"))
   expect_equal(NCOL(data_list$data), 2)
@@ -413,12 +437,14 @@ test_that("xgb.shap.data works when top_n is provided", {
 })
 
 test_that("xgb.shap.data works with subsampling", {
+  .skip_if_vcd_not_available()
   data_list <- xgb.shap.data(data = sparse_matrix, model = bst.Tree, top_n = 2, subsample = 0.8)
   expect_equal(NROW(data_list$data), as.integer(0.8 * nrow(sparse_matrix)))
   expect_equal(NROW(data_list$data), NROW(data_list$shap_contrib))
 })
 
 test_that("prepare.ggplot.shap.data works", {
+  .skip_if_vcd_not_available()
   data_list <- xgb.shap.data(data = sparse_matrix, model = bst.Tree, top_n = 2)
   plot_data <- prepare.ggplot.shap.data(data_list, normalize = TRUE)
   expect_s3_class(plot_data, "data.frame")
@@ -429,17 +455,19 @@ test_that("prepare.ggplot.shap.data works", {
 })
 
 test_that("xgb.plot.shap works", {
+  .skip_if_vcd_not_available()
   sh <- xgb.plot.shap(data = sparse_matrix, model = bst.Tree, top_n = 2, col = 4)
   expect_equal(names(sh), c("data", "shap_contrib"))
 })
 
 test_that("xgb.plot.shap.summary works", {
+  .skip_if_vcd_not_available()
   expect_silent(xgb.plot.shap.summary(data = sparse_matrix, model = bst.Tree, top_n = 2))
   expect_silent(xgb.ggplot.shap.summary(data = sparse_matrix, model = bst.Tree, top_n = 2))
 })
 
 test_that("check.deprecation works", {
-  ttt <- function(a = NNULL, DUMMY=NULL, ...) {
+  ttt <- function(a = NNULL, DUMMY = NULL, ...) {
     check.deprecation(...)
     as.list((environment()))
   }
