@@ -2,9 +2,9 @@
 Use scikit-learn regressor interface with CPU histogram tree method
 ===================================================================
 """
-from dask.distributed import Client
-from dask.distributed import LocalCluster
 from dask import array as da
+from dask.distributed import Client, LocalCluster
+
 import xgboost
 
 
