@@ -1,7 +1,12 @@
-import xgboost as xgb
+"""
+Using rmm on a single node device
+=================================
+"""
 import rmm
 from sklearn.datasets import make_classification
 
+import xgboost as xgb
+
 # Initialize RMM pool allocator
 rmm.reinitialize(pool_allocator=True)
 # Optionally force XGBoost to use RMM for all GPU memory allocation, see ./README.md
@@ -15,7 +20,8 @@ params = {
     "eta": 0.01,
     "objective": "multi:softprob",
     "num_class": 3,
-    "tree_method": "gpu_hist",
+    "tree_method": "hist",
+    "device": "cuda",
 }
 # XGBoost will automatically use the RMM pool allocator
 bst = xgb.train(params, dtrain, num_boost_round=100, evals=[(dtrain, "train")])
