@@ -1,5 +1,5 @@
-/*!
- * Copyright (c) by XGBoost Contributors 2019-2022
+/**
+ * Copyright 2019-2023 by XGBoost Contributors
  */
 #ifndef XGBOOST_JSON_H_
 #define XGBOOST_JSON_H_
@@ -13,6 +13,7 @@
 #include <map>
 #include <memory>
 #include <string>
+#include <type_traits>  // std::enable_if,std::enable_if_t
 #include <utility>
 #include <vector>
 
@@ -371,7 +372,7 @@ class Json {
   /*! \brief Use your own JsonWriter. */
   static void Dump(Json json, JsonWriter* writer);
 
-  Json() : ptr_{new JsonNull} {}
+  Json() = default;
 
   // number
   explicit Json(JsonNumber number) : ptr_{new JsonNumber(std::move(number))} {}
@@ -461,7 +462,7 @@ class Json {
   IntrusivePtr<Value> const& Ptr() const { return ptr_; }
 
  private:
-  IntrusivePtr<Value> ptr_;
+  IntrusivePtr<Value> ptr_{new JsonNull};
 };
 
 /**
@@ -595,6 +596,52 @@ using String  = JsonString;
 using Null    = JsonNull;
 
 // Utils tailored for XGBoost.
+namespace detail {
+template <typename Head>
+bool TypeCheckImpl(Json const& value) {
+  return IsA<Head>(value);
+}
+
+template <typename Head, typename... JT>
+std::enable_if_t<sizeof...(JT) != 0, bool> TypeCheckImpl(Json const& value) {
+  return IsA<Head>(value) || TypeCheckImpl<JT...>(value);
+}
+
+template <typename Head>
+std::string TypeCheckError() {
+  return "`" + Head{}.TypeStr() + "`";
+}
+
+template <typename Head, typename... JT>
+std::enable_if_t<sizeof...(JT) != 0, std::string> TypeCheckError() {
+  return "`" + Head{}.TypeStr() + "`, " + TypeCheckError<JT...>();
+}
+}  // namespace detail
+
+/**
+ * \brief Type check for JSON-based parameters
+ *
+ * \tparam JT    Expected JSON types.
+ * \param  value Value to be checked.
+ */
+template <typename... JT>
+void TypeCheck(Json const& value, StringView name) {
+  if (!detail::TypeCheckImpl<JT...>(value)) {
+    LOG(FATAL) << "Invalid type for: `" << name << "`, expecting one of the: {`"
+               << detail::TypeCheckError<JT...>() << "}, got: `" << value.GetValue().TypeStr()
+               << "`";
+  }
+}
+
+/**
+ * \brief Convert XGBoost parameter to JSON object.
+ *
+ * \tparam Parameter An instantiation of XGBoostParameter
+ *
+ * \param param Input parameter
+ *
+ * \return JSON object representing the input parameter
+ */
 template <typename Parameter>
 Object ToJson(Parameter const& param) {
   Object obj;
@@ -604,14 +651,24 @@ Object ToJson(Parameter const& param) {
   return obj;
 }
 
+/**
+ * \brief Load a XGBoost parameter from a JSON object.
+ *
+ * \tparam Parameter An instantiation of XGBoostParameter
+ *
+ * \param obj JSON object representing the parameter.
+ * \param param Output parameter.
+ *
+ * \return Unknown arguments in the JSON object.
+ */
 template <typename Parameter>
 Args FromJson(Json const& obj, Parameter* param) {
   auto const& j_param = get<Object const>(obj);
-  std::map<std::string, std::string> m;
+  Args args;
   for (auto const& kv : j_param) {
-    m[kv.first] = get<String const>(kv.second);
+    args.emplace_back(kv.first, get<String const>(kv.second));
   }
-  return param->UpdateAllowUnknown(m);
+  return param->UpdateAllowUnknown(args);
 }
 }  // namespace xgboost
 #endif  // XGBOOST_JSON_H_
