@@ -1,5 +1,5 @@
-/*!
- * Copyright 2014-2022 by Contributors
+/**
+ * Copyright 2014-2023 by XGBoost Contributors
  * \file objective.h
  * \brief interface of objective function used by xgboost.
  * \author Tianqi Chen, Kailong Chen
@@ -10,19 +10,20 @@
 #include <dmlc/registry.h>
 #include <xgboost/base.h>
 #include <xgboost/data.h>
-#include <xgboost/model.h>
-#include <xgboost/generic_parameters.h>
 #include <xgboost/host_device_vector.h>
+#include <xgboost/model.h>
 #include <xgboost/task.h>
 
-#include <vector>
-#include <utility>
-#include <string>
+#include <cstdint>  // std::int32_t
 #include <functional>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace xgboost {
 
 class RegTree;
+struct Context;
 
 /*! \brief interface of objective function */
 class ObjFunction : public Configurable {
@@ -54,6 +55,11 @@ class ObjFunction : public Configurable {
 
   /*! \return the default evaluation metric for the objective */
   virtual const char* DefaultEvalMetric() const = 0;
+  /**
+   * \brief Return the configuration for the default metric.
+   */
+  virtual Json DefaultMetricConfig() const { return Json{Null{}}; }
+
   // the following functions are optional, most of time default implementation is good enough
   /*!
    * \brief transform prediction values, this is only called when Prediction is called
@@ -93,7 +99,7 @@ class ObjFunction : public Configurable {
    * \brief Return number of targets for input matrix.  Right now XGBoost supports only
    *        multi-target regression.
    */
-  virtual uint32_t Targets(MetaInfo const& info) const {
+  virtual bst_target_t Targets(MetaInfo const& info) const {
     if (info.labels.Shape(1) > 1) {
       LOG(FATAL) << "multioutput is not supported by current objective function";
     }
@@ -110,20 +116,22 @@ class ObjFunction : public Configurable {
    *
    * \param position The leaf index for each rows.
    * \param info MetaInfo providing labels and weights.
+   * \param learning_rate The learning rate for current iteration.
    * \param prediction Model prediction after transformation.
+   * \param group_idx The group index for this tree, 0 when it's not multi-target or multi-class.
    * \param p_tree Tree that needs to be updated.
    */
   virtual void UpdateTreeLeaf(HostDeviceVector<bst_node_t> const& /*position*/,
-                              MetaInfo const& /*info*/,
+                              MetaInfo const& /*info*/, float /*learning_rate*/,
                               HostDeviceVector<float> const& /*prediction*/,
-                              RegTree* /*p_tree*/) const {}
+                              std::int32_t /*group_idx*/, RegTree* /*p_tree*/) const {}
 
   /*!
    * \brief Create an objective function according to name.
-   * \param tparam Generic parameters.
+   * \param ctx  Pointer to runtime parameters.
    * \param name Name of the objective.
    */
-  static ObjFunction* Create(const std::string& name, GenericParameter const* tparam);
+  static ObjFunction* Create(const std::string& name, Context const* ctx);
 };
 
 /*!
