@@ -20,7 +20,7 @@ import scala.collection.JavaConverters._
 
 import ml.dmlc.xgboost4j.gpu.java.CudfColumnBatch
 import ml.dmlc.xgboost4j.java.nvidia.spark.GpuColumnBatch
-import ml.dmlc.xgboost4j.scala.{Booster, DMatrix, DeviceQuantileDMatrix}
+import ml.dmlc.xgboost4j.scala.{Booster, DMatrix, QuantileDMatrix}
 import ml.dmlc.xgboost4j.scala.spark.params.XGBoostEstimatorCommon
 import ml.dmlc.xgboost4j.scala.spark.{PreXGBoost, PreXGBoostProvider, Watches, XGBoost, XGBoostClassificationModel, XGBoostClassifier, XGBoostExecutionParams, XGBoostRegressionModel, XGBoostRegressor}
 import org.apache.commons.logging.LogFactory
@@ -137,8 +137,12 @@ object GpuPreXGBoost extends PreXGBoostProvider {
     val (Seq(labelName, weightName, marginName), feturesCols, groupName, evalSets) =
       estimator match {
         case est: XGBoostEstimatorCommon =>
-          require(est.isDefined(est.treeMethod) && est.getTreeMethod.equals("gpu_hist"),
-            s"GPU train requires tree_method set to gpu_hist")
+          require(
+            est.isDefined(est.device) &&
+              (est.getDevice.equals("cuda") || est.getDevice.equals("gpu")) ||
+              est.isDefined(est.treeMethod) && est.getTreeMethod.equals("gpu_hist"),
+            s"GPU train requires `device` set to `cuda` or `gpu`."
+          )
           val groupName = estimator match {
             case regressor: XGBoostRegressor => if (regressor.isDefined(regressor.groupCol)) {
               regressor.getGroupCol } else ""
@@ -280,8 +284,7 @@ object GpuPreXGBoost extends PreXGBoostProvider {
             // - gpu id
             // - predictor: Force to gpu predictor since native doesn't save predictor.
             val gpuId = if (!isLocal) XGBoost.getGPUAddrFromResources else 0
-            booster.setParam("gpu_id", gpuId.toString)
-            booster.setParam("predictor", "gpu_predictor")
+            booster.setParam("device", s"cuda:$gpuId")
             logger.info("GPU transform on device: " + gpuId)
             boosterFlag.isGpuParamsSet = true;
           }
@@ -532,7 +535,7 @@ object GpuPreXGBoost extends PreXGBoostProvider {
   }
 
   /**
-   * Build DeviceQuantileDMatrix based on GpuColumnBatches
+   * Build QuantileDMatrix based on GpuColumnBatches
    *
    * @param iter a sequence of GpuColumnBatch
    * @param indices indicate the feature, label, weight, base margin column ids.
@@ -546,7 +549,7 @@ object GpuPreXGBoost extends PreXGBoostProvider {
       missing: Float,
       maxBin: Int): DMatrix = {
     val rapidsIterator = new RapidsIterator(iter, indices)
-    new DeviceQuantileDMatrix(rapidsIterator, missing, maxBin, 1)
+    new QuantileDMatrix(rapidsIterator, missing, maxBin, 1)
   }
 
   // zip all the Columnar RDDs into one RDD containing named column data batch.
