@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2014 by Contributors
+ Copyright (c) 2014-2022 by Contributors
 
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ private[spark] trait InferenceParams extends Params {
   final val inferBatchSize = new IntParam(this, "batchSize", "batch size of inference iteration")
 
   /** @group getParam */
-  final def getInferBatchSize: Int = ${inferBatchSize}
+  final def getInferBatchSize: Int = $(inferBatchSize)
 
   setDefault(inferBatchSize, 32 << 10)
 }
