@@ -1,21 +1,29 @@
-# coding: utf-8
-# pylint: disable=invalid-name, too-many-statements
-# pylint: disable=too-many-arguments
 """Callback library containing training routines.  See :doc:`Callback Functions
 </python/callbacks>` for a quick introduction.
 
 """
 
-from abc import ABC
 import collections
 import os
 import pickle
-from typing import Callable, List, Optional, Union, Dict, Tuple, TypeVar, cast, Sequence, Any
+from abc import ABC
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+    Union,
+    cast,
+)
+
 import numpy
 
 from . import collective
-from .core import Booster, DMatrix, XGBoostError, _get_booster_layer_trees
-
+from .core import Booster, DMatrix, XGBoostError, _parse_eval_str
 
 __all__ = [
     "TrainingCallback",
@@ -23,7 +31,7 @@ __all__ = [
     "EarlyStopping",
     "EvaluationMonitor",
     "TrainingCheckPoint",
-    "CallbackContainer"
+    "CallbackContainer",
 ]
 
 _Score = Union[float, Tuple[float, float]]
@@ -34,39 +42,37 @@ _Model = Any  # real type is Union[Booster, CVPack]; need more work
 
 # pylint: disable=unused-argument
 class TrainingCallback(ABC):
-    '''Interface for training callback.
+    """Interface for training callback.
 
     .. versionadded:: 1.3.0
 
-    '''
+    """
 
-    EvalsLog = Dict[str, Dict[str, _ScoreList]]
+    EvalsLog = Dict[str, Dict[str, _ScoreList]]  # pylint: disable=invalid-name
 
     def __init__(self) -> None:
         pass
 
     def before_training(self, model: _Model) -> _Model:
-        '''Run before training starts.'''
+        """Run before training starts."""
         return model
 
     def after_training(self, model: _Model) -> _Model:
-        '''Run after training is finished.'''
+        """Run after training is finished."""
         return model
 
     def before_iteration(self, model: _Model, epoch: int, evals_log: EvalsLog) -> bool:
-        '''Run before each iteration.  Return True when training should stop.'''
+        """Run before each iteration.  Return True when training should stop."""
         return False
 
     def after_iteration(self, model: _Model, epoch: int, evals_log: EvalsLog) -> bool:
-        '''Run after each iteration.  Return True when training should stop.'''
+        """Run after each iteration.  Return True when training should stop."""
         return False
 
 
 def _aggcv(rlist: List[str]) -> List[Tuple[str, float, float]]:
     # pylint: disable=invalid-name, too-many-locals
-    """Aggregate cross-validation results.
-
-    """
+    """Aggregate cross-validation results."""
     cvmap: Dict[Tuple[int, str], List[float]] = {}
     idx = rlist[0].split()[0]
     for line in rlist:
@@ -75,7 +81,7 @@ def _aggcv(rlist: List[str]) -> List[Tuple[str, float, float]]:
         for metric_idx, it in enumerate(arr[1:]):
             if not isinstance(it, str):
                 it = it.decode()
-            k, v = it.split(':')
+            k, v = it.split(":")
             if (metric_idx, k) not in cvmap:
                 cvmap[(metric_idx, k)] = []
             cvmap[(metric_idx, k)].append(float(v))
@@ -95,45 +101,50 @@ _ART = TypeVar("_ART")
 
 
 def _allreduce_metric(score: _ART) -> _ART:
-    '''Helper function for computing customized metric in distributed
+    """Helper function for computing customized metric in distributed
     environment.  Not strictly correct as many functions don't use mean value
     as final result.
 
-    '''
+    """
     world = collective.get_world_size()
     assert world != 0
     if world == 1:
         return score
     if isinstance(score, tuple):  # has mean and stdv
         raise ValueError(
-            'xgboost.cv function should not be used in distributed environment.')
+            "xgboost.cv function should not be used in distributed environment."
+        )
     arr = numpy.array([score])
     arr = collective.allreduce(arr, collective.Op.SUM) / world
     return arr[0]
 
 
 class CallbackContainer:
-    '''A special internal callback for invoking a list of other callbacks.
+    """A special internal callback for invoking a list of other callbacks.
 
     .. versionadded:: 1.3.0
 
-    '''
-
-    EvalsLog = TrainingCallback.EvalsLog
+    """
 
     def __init__(
         self,
         callbacks: Sequence[TrainingCallback],
-        metric: Callable = None,
+        metric: Optional[Callable] = None,
         output_margin: bool = True,
-        is_cv: bool = False
+        is_cv: bool = False,
     ) -> None:
         self.callbacks = set(callbacks)
-        if metric is not None:
-            msg = 'metric must be callable object for monitoring.  For ' + \
-                'builtin metrics, passing them in training parameter' + \
-                ' will invoke monitor automatically.'
-            assert callable(metric), msg
+        for cb in callbacks:
+            if not isinstance(cb, TrainingCallback):
+                raise TypeError("callback must be an instance of `TrainingCallback`.")
+
+        msg = (
+            "metric must be callable object for monitoring.  For builtin metrics"
+            ", passing them in training parameter invokes monitor automatically."
+        )
+        if metric is not None and not callable(metric):
+            raise TypeError(msg)
+
         self.metric = metric
         self.history: TrainingCallback.EvalsLog = collections.OrderedDict()
         self._output_margin = output_margin
@@ -143,10 +154,10 @@ class CallbackContainer:
             self.aggregated_cv = None
 
     def before_training(self, model: _Model) -> _Model:
-        '''Function called before training.'''
+        """Function called before training."""
         for c in self.callbacks:
             model = c.before_training(model=model)
-            msg = 'before_training should return the model'
+            msg = "before_training should return the model"
             if self.is_cv:
                 assert isinstance(model.cvfolds, list), msg
             else:
@@ -154,46 +165,33 @@ class CallbackContainer:
         return model
 
     def after_training(self, model: _Model) -> _Model:
-        '''Function called after training.'''
+        """Function called after training."""
         for c in self.callbacks:
             model = c.after_training(model=model)
-            msg = 'after_training should return the model'
+            msg = "after_training should return the model"
             if self.is_cv:
                 assert isinstance(model.cvfolds, list), msg
             else:
                 assert isinstance(model, Booster), msg
 
-        if not self.is_cv:
-            num_parallel_tree, _ = _get_booster_layer_trees(model)
-            if model.attr('best_score') is not None:
-                model.best_score = float(cast(str, model.attr('best_score')))
-                model.best_iteration = int(cast(str, model.attr('best_iteration')))
-                # num_class is handled internally
-                model.set_attr(
-                    best_ntree_limit=str((model.best_iteration + 1) * num_parallel_tree)
-                )
-                model.best_ntree_limit = int(cast(str, model.attr("best_ntree_limit")))
-            else:
-                # Due to compatibility with version older than 1.4, these attributes are
-                # added to Python object even if early stopping is not used.
-                model.best_iteration = model.num_boosted_rounds() - 1
-                model.set_attr(best_iteration=str(model.best_iteration))
-                model.best_ntree_limit = (model.best_iteration + 1) * num_parallel_tree
-                model.set_attr(best_ntree_limit=str(model.best_ntree_limit))
-
         return model
 
     def before_iteration(
-        self, model: _Model, epoch: int, dtrain: DMatrix, evals: Optional[List[Tuple[DMatrix, str]]]
+        self,
+        model: _Model,
+        epoch: int,
+        dtrain: DMatrix,
+        evals: Optional[List[Tuple[DMatrix, str]]],
     ) -> bool:
-        '''Function called before training iteration.'''
-        return any(c.before_iteration(model, epoch, self.history)
-                   for c in self.callbacks)
+        """Function called before training iteration."""
+        return any(
+            c.before_iteration(model, epoch, self.history) for c in self.callbacks
+        )
 
     def _update_history(
         self,
         score: Union[List[Tuple[str, float]], List[Tuple[str, float, float]]],
-        epoch: int
+        epoch: int,
     ) -> None:
         for d in score:
             name: str = d[0]
@@ -203,9 +201,9 @@ class CallbackContainer:
                 x: _Score = (s, std)
             else:
                 x = s
-            splited_names = name.split('-')
+            splited_names = name.split("-")
             data_name = splited_names[0]
-            metric_name = '-'.join(splited_names[1:])
+            metric_name = "-".join(splited_names[1:])
             x = _allreduce_metric(x)
             if data_name not in self.history:
                 self.history[data_name] = collections.OrderedDict()
@@ -227,7 +225,7 @@ class CallbackContainer:
         dtrain: DMatrix,
         evals: Optional[List[Tuple[DMatrix, str]]],
     ) -> bool:
-        '''Function called after training iteration.'''
+        """Function called after training iteration."""
         if self.is_cv:
             scores = model.eval(epoch, self.metric, self._output_margin)
             scores = _aggcv(scores)
@@ -236,16 +234,11 @@ class CallbackContainer:
         else:
             evals = [] if evals is None else evals
             for _, name in evals:
-                assert name.find('-') == -1, 'Dataset name should not contain `-`'
+                assert name.find("-") == -1, "Dataset name should not contain `-`"
             score: str = model.eval_set(evals, epoch, self.metric, self._output_margin)
-            splited = score.split()[1:]  # into datasets
-            # split up `test-error:0.1234`
-            metric_score_str = [tuple(s.split(':')) for s in splited]
-            # convert to float
-            metric_score = [(n, float(s)) for n, s in metric_score_str]
+            metric_score = _parse_eval_str(score)
             self._update_history(metric_score, epoch)
-        ret = any(c.after_iteration(model, epoch, self.history)
-                  for c in self.callbacks)
+        ret = any(c.after_iteration(model, epoch, self.history) for c in self.callbacks)
         return ret
 
 
@@ -268,9 +261,14 @@ class LearningRateScheduler(TrainingCallback):
     def __init__(
         self, learning_rates: Union[Callable[[int], float], Sequence[float]]
     ) -> None:
-        assert callable(learning_rates) or isinstance(
+        if not callable(learning_rates) and not isinstance(
             learning_rates, collections.abc.Sequence
-        )
+        ):
+            raise TypeError(
+                "Invalid learning rates, expecting callable or sequence, got: "
+                f"{type(learning_rates)}"
+            )
+
         if callable(learning_rates):
             self.learning_rates = learning_rates
         else:
@@ -303,25 +301,31 @@ class EarlyStopping(TrainingCallback):
     save_best :
         Whether training should return the best model or the last model.
     min_delta :
-        Minimum absolute change in score to be qualified as an improvement.
 
         .. versionadded:: 1.5.0
 
-        .. code-block:: python
+        Minimum absolute change in score to be qualified as an improvement.
 
-            clf = xgboost.XGBClassifier(tree_method="gpu_hist")
-            es = xgboost.callback.EarlyStopping(
-                rounds=2,
-                abs_tol=1e-3,
-                save_best=True,
-                maximize=False,
-                data_name="validation_0",
-                metric_name="mlogloss",
-            )
+    Examples
+    --------
 
-            X, y = load_digits(return_X_y=True)
-            clf.fit(X, y, eval_set=[(X, y)], callbacks=[es])
+    .. code-block:: python
+
+        es = xgboost.callback.EarlyStopping(
+            rounds=2,
+            min_delta=1e-3,
+            save_best=True,
+            maximize=False,
+            data_name="validation_0",
+            metric_name="mlogloss",
+        )
+        clf = xgboost.XGBClassifier(tree_method="hist", device="cuda", callbacks=[es])
+
+        X, y = load_digits(return_X_y=True)
+        clf.fit(X, y, eval_set=[(X, y)])
     """
+
+    # pylint: disable=too-many-arguments
     def __init__(
         self,
         rounds: int,
@@ -329,7 +333,7 @@ class EarlyStopping(TrainingCallback):
         data_name: Optional[str] = None,
         maximize: Optional[bool] = None,
         save_best: Optional[bool] = False,
-        min_delta: float = 0.0
+        min_delta: float = 0.0,
     ) -> None:
         self.data = data_name
         self.metric_name = metric_name
@@ -353,24 +357,34 @@ class EarlyStopping(TrainingCallback):
     def _update_rounds(
         self, score: _Score, name: str, metric: str, model: _Model, epoch: int
     ) -> bool:
-        def get_s(x: _Score) -> float:
+        def get_s(value: _Score) -> float:
             """get score if it's cross validation history."""
-            return x[0] if isinstance(x, tuple) else x
+            return value[0] if isinstance(value, tuple) else value
 
         def maximize(new: _Score, best: _Score) -> bool:
             """New score should be greater than the old one."""
             return numpy.greater(get_s(new) - self._min_delta, get_s(best))
 
         def minimize(new: _Score, best: _Score) -> bool:
-            """New score should be smaller than the old one."""
+            """New score should be lesser than the old one."""
             return numpy.greater(get_s(best) - self._min_delta, get_s(new))
 
         if self.maximize is None:
             # Just to be compatibility with old behavior before 1.3.  We should let
             # user to decide.
-            maximize_metrics = ('auc', 'aucpr', 'map', 'ndcg', 'auc@',
-                                'aucpr@', 'map@', 'ndcg@')
-            if metric != 'mape' and any(metric.startswith(x) for x in maximize_metrics):
+            maximize_metrics = (
+                "auc",
+                "aucpr",
+                "pre",
+                "pre@",
+                "map",
+                "ndcg",
+                "auc@",
+                "aucpr@",
+                "map@",
+                "ndcg@",
+            )
+            if metric != "mape" and any(metric.startswith(x) for x in maximize_metrics):
                 self.maximize = True
             else:
                 self.maximize = False
@@ -380,8 +394,6 @@ class EarlyStopping(TrainingCallback):
         else:
             improve_op = minimize
 
-        assert improve_op
-
         if not self.stopping_history:  # First round
             self.current_rounds = 0
             self.stopping_history[name] = {}
@@ -405,62 +417,77 @@ class EarlyStopping(TrainingCallback):
             return True
         return False
 
-    def after_iteration(self, model: _Model, epoch: int,
-                        evals_log: TrainingCallback.EvalsLog) -> bool:
+    def after_iteration(
+        self, model: _Model, epoch: int, evals_log: TrainingCallback.EvalsLog
+    ) -> bool:
         epoch += self.starting_round  # training continuation
-        msg = 'Must have at least 1 validation dataset for early stopping.'
-        assert len(evals_log.keys()) >= 1, msg
-        data_name = ''
+        msg = "Must have at least 1 validation dataset for early stopping."
+        if len(evals_log.keys()) < 1:
+            raise ValueError(msg)
+
+        # Get data name
         if self.data:
-            for d, _ in evals_log.items():
-                if d == self.data:
-                    data_name = d
-            if not data_name:
-                raise ValueError('No dataset named:', self.data)
+            data_name = self.data
         else:
             # Use the last one as default.
             data_name = list(evals_log.keys())[-1]
-        assert isinstance(data_name, str) and data_name
+        if data_name not in evals_log:
+            raise ValueError(f"No dataset named: {data_name}")
+
+        if not isinstance(data_name, str):
+            raise TypeError(
+                f"The name of the dataset should be a string. Got: {type(data_name)}"
+            )
         data_log = evals_log[data_name]
 
-        # Filter out scores that can not be used for early stopping.
+        # Get metric name
         if self.metric_name:
             metric_name = self.metric_name
         else:
             # Use last metric by default.
-            assert isinstance(data_log, collections.OrderedDict)
             metric_name = list(data_log.keys())[-1]
+        if metric_name not in data_log:
+            raise ValueError(f"No metric named: {metric_name}")
+
+        # The latest score
         score = data_log[metric_name][-1]
         return self._update_rounds(score, data_name, metric_name, model, epoch)
 
     def after_training(self, model: _Model) -> _Model:
+        if not self.save_best:
+            return model
+
         try:
-            if self.save_best:
-                model = model[: int(model.attr("best_iteration")) + 1]
+            best_iteration = model.best_iteration
+            best_score = model.best_score
+            assert best_iteration is not None and best_score is not None
+            model = model[: best_iteration + 1]
+            model.best_iteration = best_iteration
+            model.best_score = best_score
         except XGBoostError as e:
             raise XGBoostError(
-                "`save_best` is not applicable to current booster"
+                "`save_best` is not applicable to the current booster"
             ) from e
+
         return model
 
 
 class EvaluationMonitor(TrainingCallback):
-    '''Print the evaluation result at each iteration.
+    """Print the evaluation result at each iteration.
 
     .. versionadded:: 1.3.0
 
     Parameters
     ----------
 
-    metric :
-        Extra user defined metric.
     rank :
         Which worker should be used for printing the result.
     period :
         How many epoches between printing.
     show_stdv :
         Used in cv to show standard deviation.  Users should not specify it.
-    '''
+    """
+
     def __init__(self, rank: int = 0, period: int = 1, show_stdv: bool = False) -> None:
         self.printer_rank = rank
         self.show_stdv = show_stdv
@@ -479,12 +506,13 @@ class EvaluationMonitor(TrainingCallback):
             msg = f"\t{data + '-' + metric}:{score:.5f}"
         return msg
 
-    def after_iteration(self, model: _Model, epoch: int,
-                        evals_log: TrainingCallback.EvalsLog) -> bool:
+    def after_iteration(
+        self, model: _Model, epoch: int, evals_log: TrainingCallback.EvalsLog
+    ) -> bool:
         if not evals_log:
             return False
 
-        msg: str = f'[{epoch}]'
+        msg: str = f"[{epoch}]"
         if collective.get_rank() == self.printer_rank:
             for data, metric in evals_log.items():
                 for metric_name, log in metric.items():
@@ -495,7 +523,7 @@ class EvaluationMonitor(TrainingCallback):
                     else:
                         score = log[-1]
                     msg += self._fmt_metric(data, metric_name, score, stdv)
-            msg += '\n'
+            msg += "\n"
 
             if (epoch % self.period) == 0 or self.period == 1:
                 collective.communicator_print(msg)
@@ -512,7 +540,7 @@ class EvaluationMonitor(TrainingCallback):
 
 
 class TrainingCheckPoint(TrainingCallback):
-    '''Checkpointing operation.
+    """Checkpointing operation.
 
     .. versionadded:: 1.3.0
 
@@ -531,13 +559,14 @@ class TrainingCheckPoint(TrainingCallback):
         Interval of checkpointing.  Checkpointing is slow so setting a larger number can
         reduce performance hit.
 
-    '''
+    """
+
     def __init__(
         self,
         directory: Union[str, os.PathLike],
-        name: str = 'model',
+        name: str = "model",
         as_pickle: bool = False,
-        iterations: int = 100
+        iterations: int = 100,
     ) -> None:
         self._path = os.fspath(directory)
         self._name = name
@@ -546,15 +575,21 @@ class TrainingCheckPoint(TrainingCallback):
         self._epoch = 0
         super().__init__()
 
-    def after_iteration(self, model: _Model, epoch: int,
-                        evals_log: TrainingCallback.EvalsLog) -> bool:
+    def after_iteration(
+        self, model: _Model, epoch: int, evals_log: TrainingCallback.EvalsLog
+    ) -> bool:
         if self._epoch == self._iterations:
-            path = os.path.join(self._path, self._name + '_' + str(epoch) +
-                                ('.pkl' if self._as_pickle else '.json'))
+            path = os.path.join(
+                self._path,
+                self._name
+                + "_"
+                + str(epoch)
+                + (".pkl" if self._as_pickle else ".json"),
+            )
             self._epoch = 0
             if collective.get_rank() == 0:
                 if self._as_pickle:
-                    with open(path, 'wb') as fd:
+                    with open(path, "wb") as fd:
                         pickle.dump(model, fd)
                 else:
                     model.save_model(path)
