@@ -1,6 +1,4 @@
-# type: ignore
-"""PySpark XGBoost integration interface
-"""
+"""PySpark XGBoost integration interface"""
 
 try:
     import pyspark
@@ -11,6 +9,7 @@ from .estimator import (
     SparkXGBClassifier,
     SparkXGBClassifierModel,
     SparkXGBRanker,
+    SparkXGBRankerModel,
     SparkXGBRegressor,
     SparkXGBRegressorModel,
 )
@@ -21,4 +20,5 @@ __all__ = [
     "SparkXGBRegressor",
     "SparkXGBRegressorModel",
     "SparkXGBRanker",
+    "SparkXGBRankerModel",
 ]
