@@ -1,37 +1,40 @@
-# coding: utf-8
 # pylint: disable=too-many-locals, too-many-arguments, invalid-name
 # pylint: disable=too-many-branches, too-many-statements
 """Training Library containing training routines."""
 import copy
 import os
 import warnings
-from typing import Optional, Dict, Any, Union, Tuple, Sequence
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union, cast
 
 import numpy as np
-from .core import Booster, DMatrix, XGBoostError, _deprecate_positional_args
-from .core import Metric, Objective
-from .compat import (SKLEARN_INSTALLED, XGBStratifiedKFold)
-from . import callback
 
+from ._typing import BoosterParam, Callable, FPreProcCallable
+from .callback import (
+    CallbackContainer,
+    EarlyStopping,
+    EvaluationMonitor,
+    TrainingCallback,
+)
+from .compat import SKLEARN_INSTALLED, DataFrame, XGBStratifiedKFold
+from .core import (
+    Booster,
+    DMatrix,
+    Metric,
+    Objective,
+    XGBoostError,
+    _deprecate_positional_args,
+)
 
-def _assert_new_callback(
-    callbacks: Optional[Sequence[callback.TrainingCallback]]
-) -> None:
-    is_new_callback: bool = not callbacks or all(
-        isinstance(c, callback.TrainingCallback) for c in callbacks
-    )
-    if not is_new_callback:
-        link = "https://xgboost.readthedocs.io/en/latest/python/callbacks.html"
-        raise ValueError(
-            f"Old style callback was removed in version 1.6.  See: {link}."
-        )
+_CVFolds = Sequence["CVPack"]
 
 
 def _configure_custom_metric(
     feval: Optional[Metric], custom_metric: Optional[Metric]
 ) -> Optional[Metric]:
     if feval is not None:
-        link = "https://xgboost.readthedocs.io/en/latest/tutorials/custom_metric_obj.html"
+        link = (
+            "https://xgboost.readthedocs.io/en/latest/tutorials/custom_metric_obj.html"
+        )
         warnings.warn(
             "`feval` is deprecated, use `custom_metric` instead.  They have "
             "different behavior when custom objective is also used."
@@ -56,10 +59,10 @@ def train(
     feval: Optional[Metric] = None,
     maximize: Optional[bool] = None,
     early_stopping_rounds: Optional[int] = None,
-    evals_result: callback.TrainingCallback.EvalsLog = None,
+    evals_result: Optional[TrainingCallback.EvalsLog] = None,
     verbose_eval: Optional[Union[bool, int]] = True,
     xgb_model: Optional[Union[str, os.PathLike, Booster, bytearray]] = None,
-    callbacks: Optional[Sequence[callback.TrainingCallback]] = None,
+    callbacks: Optional[Sequence[TrainingCallback]] = None,
     custom_metric: Optional[Metric] = None,
 ) -> Booster:
     """Train a booster with given parameters.
@@ -81,7 +84,7 @@ def train(
     feval :
         .. deprecated:: 1.6.0
             Use `custom_metric` instead.
-    maximize : bool
+    maximize :
         Whether to maximize feval.
     early_stopping_rounds :
         Activates early stopping. Validation metric needs to improve at least once in
@@ -156,15 +159,12 @@ def train(
     bst = Booster(params, [dtrain] + [d[0] for d in evals], model_file=xgb_model)
     start_iteration = 0
 
-    _assert_new_callback(callbacks)
     if verbose_eval:
         verbose_eval = 1 if verbose_eval is True else verbose_eval
-        callbacks.append(callback.EvaluationMonitor(period=verbose_eval))
+        callbacks.append(EvaluationMonitor(period=verbose_eval))
     if early_stopping_rounds:
-        callbacks.append(
-            callback.EarlyStopping(rounds=early_stopping_rounds, maximize=maximize)
-        )
-    cb_container = callback.CallbackContainer(
+        callbacks.append(EarlyStopping(rounds=early_stopping_rounds, maximize=maximize))
+    cb_container = CallbackContainer(
         callbacks,
         metric=metric_fn,
         # For old `feval` parameter, the behavior is unchanged.  For the new
@@ -193,88 +193,114 @@ def train(
 
 
 class CVPack:
-    """"Auxiliary datastruct to hold one fold of CV."""
-    def __init__(self, dtrain, dtest, param):
-        """"Initialize the CVPack"""
+    """ "Auxiliary datastruct to hold one fold of CV."""
+
+    def __init__(
+        self, dtrain: DMatrix, dtest: DMatrix, param: Optional[Union[Dict, List]]
+    ) -> None:
+        """ "Initialize the CVPack"""
         self.dtrain = dtrain
         self.dtest = dtest
-        self.watchlist = [(dtrain, 'train'), (dtest, 'test')]
+        self.watchlist = [(dtrain, "train"), (dtest, "test")]
         self.bst = Booster(param, [dtrain, dtest])
 
-    def __getattr__(self, name):
-        def _inner(*args, **kwargs):
+    def __getattr__(self, name: str) -> Callable:
+        def _inner(*args: Any, **kwargs: Any) -> Any:
             return getattr(self.bst, name)(*args, **kwargs)
+
         return _inner
 
-    def update(self, iteration, fobj):
-        """"Update the boosters for one iteration"""
+    def update(self, iteration: int, fobj: Optional[Objective]) -> None:
+        """ "Update the boosters for one iteration"""
         self.bst.update(self.dtrain, iteration, fobj)
 
-    def eval(self, iteration, feval, output_margin):
-        """"Evaluate the CVPack for one iteration."""
+    def eval(self, iteration: int, feval: Optional[Metric], output_margin: bool) -> str:
+        """ "Evaluate the CVPack for one iteration."""
         return self.bst.eval_set(self.watchlist, iteration, feval, output_margin)
 
 
 class _PackedBooster:
-    def __init__(self, cvfolds) -> None:
+    def __init__(self, cvfolds: _CVFolds) -> None:
         self.cvfolds = cvfolds
 
-    def update(self, iteration, obj):
-        '''Iterate through folds for update'''
+    def update(self, iteration: int, obj: Optional[Objective]) -> None:
+        """Iterate through folds for update"""
         for fold in self.cvfolds:
             fold.update(iteration, obj)
 
-    def eval(self, iteration, feval, output_margin):
-        '''Iterate through folds for eval'''
+    def eval(
+        self, iteration: int, feval: Optional[Metric], output_margin: bool
+    ) -> List[str]:
+        """Iterate through folds for eval"""
         result = [f.eval(iteration, feval, output_margin) for f in self.cvfolds]
         return result
 
-    def set_attr(self, **kwargs):
-        '''Iterate through folds for setting attributes'''
+    def set_attr(self, **kwargs: Optional[Any]) -> Any:
+        """Iterate through folds for setting attributes"""
         for f in self.cvfolds:
             f.bst.set_attr(**kwargs)
 
-    def attr(self, key):
-        '''Redirect to booster attr.'''
+    def attr(self, key: str) -> Optional[str]:
+        """Redirect to booster attr."""
         return self.cvfolds[0].bst.attr(key)
 
-    def set_param(self, params, value=None):
+    def set_param(
+        self,
+        params: Union[Dict, Iterable[Tuple[str, Any]], str],
+        value: Optional[str] = None,
+    ) -> None:
         """Iterate through folds for set_param"""
         for f in self.cvfolds:
             f.bst.set_param(params, value)
 
-    def num_boosted_rounds(self):
-        '''Number of boosted rounds.'''
+    def num_boosted_rounds(self) -> int:
+        """Number of boosted rounds."""
         return self.cvfolds[0].num_boosted_rounds()
 
     @property
-    def best_iteration(self):
-        '''Get best_iteration'''
-        return int(self.cvfolds[0].bst.attr("best_iteration"))
+    def best_iteration(self) -> int:
+        """Get best_iteration"""
+        return int(cast(int, self.cvfolds[0].bst.attr("best_iteration")))
+
+    @best_iteration.setter
+    def best_iteration(self, iteration: int) -> None:
+        """Get best_iteration"""
+        self.set_attr(best_iteration=iteration)
 
     @property
-    def best_score(self):
+    def best_score(self) -> float:
         """Get best_score."""
-        return float(self.cvfolds[0].bst.attr("best_score"))
+        return float(cast(float, self.cvfolds[0].bst.attr("best_score")))
+
+    @best_score.setter
+    def best_score(self, score: float) -> None:
+        self.set_attr(best_score=score)
 
 
-def groups_to_rows(groups, boundaries):
+def groups_to_rows(groups: List[np.ndarray], boundaries: np.ndarray) -> np.ndarray:
     """
     Given group row boundaries, convert ground indexes to row indexes
     :param groups: list of groups for testing
     :param boundaries: rows index limits of each group
     :return: row in group
     """
-    return np.concatenate([np.arange(boundaries[g], boundaries[g+1]) for g in groups])
+    return np.concatenate([np.arange(boundaries[g], boundaries[g + 1]) for g in groups])
 
 
-def mkgroupfold(dall, nfold, param, evals=(), fpreproc=None, shuffle=True):
+def mkgroupfold(
+    dall: DMatrix,
+    nfold: int,
+    param: BoosterParam,
+    evals: Sequence[str] = (),
+    fpreproc: Optional[FPreProcCallable] = None,
+    shuffle: bool = True,
+) -> List[CVPack]:
     """
     Make n folds for cross-validation maintaining groups
     :return: cross-validation folds
     """
     # we have groups for pairwise ranking... get a list of the group indexes
-    group_boundaries = dall.get_uint_info('group_ptr')
+    group_boundaries = dall.get_uint_info("group_ptr")
     group_sizes = np.diff(group_boundaries)
 
     if shuffle is True:
@@ -284,11 +310,17 @@ def mkgroupfold(dall, nfold, param, evals=(), fpreproc=None, shuffle=True):
     # list by fold of test group indexes
     out_group_idset = np.array_split(idx, nfold)
     # list by fold of train group indexes
-    in_group_idset = [np.concatenate([out_group_idset[i] for i in range(nfold) if k != i])
-                      for k in range(nfold)]
+    in_group_idset = [
+        np.concatenate([out_group_idset[i] for i in range(nfold) if k != i])
+        for k in range(nfold)
+    ]
     # from the group indexes, convert them to row indexes
-    in_idset = [groups_to_rows(in_groups, group_boundaries) for in_groups in in_group_idset]
-    out_idset = [groups_to_rows(out_groups, group_boundaries) for out_groups in out_group_idset]
+    in_idset = [
+        groups_to_rows(in_groups, group_boundaries) for in_groups in in_group_idset
+    ]
+    out_idset = [
+        groups_to_rows(out_groups, group_boundaries) for out_groups in out_group_idset
+    ]
 
     # build the folds by taking the appropriate slices
     ret = []
@@ -303,13 +335,22 @@ def mkgroupfold(dall, nfold, param, evals=(), fpreproc=None, shuffle=True):
             dtrain, dtest, tparam = fpreproc(dtrain, dtest, param.copy())
         else:
             tparam = param
-        plst = list(tparam.items()) + [('eval_metric', itm) for itm in evals]
+        plst = list(tparam.items()) + [("eval_metric", itm) for itm in evals]
         ret.append(CVPack(dtrain, dtest, plst))
     return ret
 
 
-def mknfold(dall, nfold, param, seed, evals=(), fpreproc=None, stratified=False,
-            folds=None, shuffle=True):
+def mknfold(
+    dall: DMatrix,
+    nfold: int,
+    param: BoosterParam,
+    seed: int,
+    evals: Sequence[str] = (),
+    fpreproc: Optional[FPreProcCallable] = None,
+    stratified: Optional[bool] = False,
+    folds: Optional[XGBStratifiedKFold] = None,
+    shuffle: bool = True,
+) -> List[CVPack]:
     """
     Make an n-fold list of CVPack from random indices.
     """
@@ -318,16 +359,20 @@ def mknfold(dall, nfold, param, seed, evals=(), fpreproc=None, stratified=False,
 
     if stratified is False and folds is None:
         # Do standard k-fold cross validation. Automatically determine the folds.
-        if len(dall.get_uint_info('group_ptr')) > 1:
-            return mkgroupfold(dall, nfold, param, evals=evals, fpreproc=fpreproc, shuffle=shuffle)
+        if len(dall.get_uint_info("group_ptr")) > 1:
+            return mkgroupfold(
+                dall, nfold, param, evals=evals, fpreproc=fpreproc, shuffle=shuffle
+            )
 
         if shuffle is True:
             idx = np.random.permutation(dall.num_row())
         else:
             idx = np.arange(dall.num_row())
         out_idset = np.array_split(idx, nfold)
-        in_idset = [np.concatenate([out_idset[i] for i in range(nfold) if k != i])
-                    for k in range(nfold)]
+        in_idset = [
+            np.concatenate([out_idset[i] for i in range(nfold) if k != i])
+            for k in range(nfold)
+        ]
     elif folds is not None:
         # Use user specified custom split using indices
         try:
@@ -357,16 +402,32 @@ def mknfold(dall, nfold, param, seed, evals=(), fpreproc=None, stratified=False,
             dtrain, dtest, tparam = fpreproc(dtrain, dtest, param.copy())
         else:
             tparam = param
-        plst = list(tparam.items()) + [('eval_metric', itm) for itm in evals]
+        plst = list(tparam.items()) + [("eval_metric", itm) for itm in evals]
         ret.append(CVPack(dtrain, dtest, plst))
     return ret
 
 
-def cv(params, dtrain, num_boost_round=10, nfold=3, stratified=False, folds=None,
-       metrics=(), obj: Optional[Objective] = None,
-       feval=None, maximize=None, early_stopping_rounds=None,
-       fpreproc=None, as_pandas=True, verbose_eval=None, show_stdv=True,
-       seed=0, callbacks=None, shuffle=True, custom_metric: Optional[Metric] = None):
+def cv(
+    params: BoosterParam,
+    dtrain: DMatrix,
+    num_boost_round: int = 10,
+    nfold: int = 3,
+    stratified: bool = False,
+    folds: XGBStratifiedKFold = None,
+    metrics: Sequence[str] = (),
+    obj: Optional[Objective] = None,
+    feval: Optional[Metric] = None,
+    maximize: Optional[bool] = None,
+    early_stopping_rounds: Optional[int] = None,
+    fpreproc: Optional[FPreProcCallable] = None,
+    as_pandas: bool = True,
+    verbose_eval: Optional[Union[int, bool]] = None,
+    show_stdv: bool = True,
+    seed: int = 0,
+    callbacks: Optional[Sequence[TrainingCallback]] = None,
+    shuffle: bool = True,
+    custom_metric: Optional[Metric] = None,
+) -> Union[Dict[str, float], DataFrame]:
     # pylint: disable = invalid-name
     """Cross-validation with given parameters.
 
@@ -456,47 +517,44 @@ def cv(params, dtrain, num_boost_round=10, nfold=3, stratified=False, folds=None
     evaluation history : list(string)
     """
     if stratified is True and not SKLEARN_INSTALLED:
-        raise XGBoostError('sklearn needs to be installed in order to use stratified cv')
+        raise XGBoostError(
+            "sklearn needs to be installed in order to use stratified cv"
+        )
 
     if isinstance(metrics, str):
         metrics = [metrics]
 
+    params = params.copy()
     if isinstance(params, list):
-        _metrics = [x[1] for x in params if x[0] == 'eval_metric']
+        _metrics = [x[1] for x in params if x[0] == "eval_metric"]
         params = dict(params)
-        if 'eval_metric' in params:
-            params['eval_metric'] = _metrics
-    else:
-        params = dict((k, v) for k, v in params.items())
+        if "eval_metric" in params:
+            params["eval_metric"] = _metrics
 
-    if (not metrics) and 'eval_metric' in params:
-        if isinstance(params['eval_metric'], list):
-            metrics = params['eval_metric']
+    if (not metrics) and "eval_metric" in params:
+        if isinstance(params["eval_metric"], list):
+            metrics = params["eval_metric"]
         else:
-            metrics = [params['eval_metric']]
+            metrics = [params["eval_metric"]]
 
     params.pop("eval_metric", None)
 
-    results = {}
-    cvfolds = mknfold(dtrain, nfold, params, seed, metrics, fpreproc,
-                      stratified, folds, shuffle)
+    results: Dict[str, List[float]] = {}
+    cvfolds = mknfold(
+        dtrain, nfold, params, seed, metrics, fpreproc, stratified, folds, shuffle
+    )
 
     metric_fn = _configure_custom_metric(feval, custom_metric)
 
     # setup callbacks
     callbacks = [] if callbacks is None else copy.copy(list(callbacks))
-    _assert_new_callback(callbacks)
 
     if verbose_eval:
         verbose_eval = 1 if verbose_eval is True else verbose_eval
-        callbacks.append(
-            callback.EvaluationMonitor(period=verbose_eval, show_stdv=show_stdv)
-        )
+        callbacks.append(EvaluationMonitor(period=verbose_eval, show_stdv=show_stdv))
     if early_stopping_rounds:
-        callbacks.append(
-            callback.EarlyStopping(rounds=early_stopping_rounds, maximize=maximize)
-        )
-    callbacks = callback.CallbackContainer(
+        callbacks.append(EarlyStopping(rounds=early_stopping_rounds, maximize=maximize))
+    callbacks_container = CallbackContainer(
         callbacks,
         metric=metric_fn,
         is_cv=True,
@@ -504,34 +562,35 @@ def cv(params, dtrain, num_boost_round=10, nfold=3, stratified=False, folds=None
     )
 
     booster = _PackedBooster(cvfolds)
-    callbacks.before_training(booster)
+    callbacks_container.before_training(booster)
 
     for i in range(num_boost_round):
-        if callbacks.before_iteration(booster, i, dtrain, None):
+        if callbacks_container.before_iteration(booster, i, dtrain, None):
             break
         booster.update(i, obj)
 
-        should_break = callbacks.after_iteration(booster, i, dtrain, None)
-        res = callbacks.aggregated_cv
-        for key, mean, std in res:
-            if key + '-mean' not in results:
-                results[key + '-mean'] = []
-            if key + '-std' not in results:
-                results[key + '-std'] = []
-            results[key + '-mean'].append(mean)
-            results[key + '-std'].append(std)
+        should_break = callbacks_container.after_iteration(booster, i, dtrain, None)
+        res = callbacks_container.aggregated_cv
+        for key, mean, std in cast(List[Tuple[str, float, float]], res):
+            if key + "-mean" not in results:
+                results[key + "-mean"] = []
+            if key + "-std" not in results:
+                results[key + "-std"] = []
+            results[key + "-mean"].append(mean)
+            results[key + "-std"].append(std)
 
         if should_break:
             for k in results.keys():  # pylint: disable=consider-iterating-dictionary
-                results[k] = results[k][:(booster.best_iteration + 1)]
+                results[k] = results[k][: (booster.best_iteration + 1)]
             break
     if as_pandas:
         try:
             import pandas as pd
+
             results = pd.DataFrame.from_dict(results)
         except ImportError:
             pass
 
-    callbacks.after_training(booster)
+    callbacks_container.after_training(booster)
 
     return results
