@@ -6,25 +6,26 @@
 #define XGBOOST_DATA_ADAPTER_H_
 #include <dmlc/data.h>
 
-#include <cstddef>
+#include <algorithm>
+#include <cstddef>  // for size_t
 #include <functional>
 #include <limits>
+#include <map>
 #include <memory>
 #include <string>
-#include <utility>
+#include <utility>  // std::move
 #include <vector>
-#include <map>
-#include <algorithm>
 
-#include "xgboost/logging.h"
+#include "../c_api/c_api_error.h"
+#include "../common/error_msg.h"  // for MaxFeatureSize
+#include "../common/math.h"
+#include "array_interface.h"
+#include "arrow-cdi.h"
 #include "xgboost/base.h"
 #include "xgboost/data.h"
+#include "xgboost/logging.h"
 #include "xgboost/span.h"
-
-#include "array_interface.h"
-#include "../c_api/c_api_error.h"
-#include "../common/math.h"
-#include "arrow-cdi.h"
+#include "xgboost/string_view.h"
 
 namespace xgboost {
 namespace data {
@@ -300,9 +301,9 @@ class ArrayAdapter : public detail::SingleBatchDataIter<ArrayAdapterBatch> {
     array_interface_ = ArrayInterface<2>(get<Object const>(j));
     batch_ = ArrayAdapterBatch{array_interface_};
   }
-  ArrayAdapterBatch const& Value() const override { return batch_; }
-  size_t NumRows() const { return array_interface_.Shape(0); }
-  size_t NumColumns() const { return array_interface_.Shape(1); }
+  [[nodiscard]] ArrayAdapterBatch const& Value() const override { return batch_; }
+  [[nodiscard]] std::size_t NumRows() const { return array_interface_.Shape(0); }
+  [[nodiscard]] std::size_t NumColumns() const { return array_interface_.Shape(1); }
 
  private:
   ArrayAdapterBatch batch_;
@@ -327,8 +328,8 @@ class CSRArrayAdapterBatch : public detail::NoMetaInfo {
         : indices_{std::move(indices)}, values_{std::move(values)}, ridx_{ridx},
           offset_{offset} {}
 
-    COOTuple GetElement(size_t idx) const {
-      return {ridx_, TypedIndex<size_t, 1>{indices_}(offset_ + idx), values_(offset_ + idx)};
+    COOTuple GetElement(std::size_t idx) const {
+      return {ridx_, TypedIndex<std::size_t, 1>{indices_}(offset_ + idx), values_(offset_ + idx)};
     }
 
     size_t Size() const {
@@ -472,17 +473,81 @@ class CSCAdapter : public detail::SingleBatchDataIter<CSCAdapterBatch> {
   size_t num_columns_;
 };
 
-class DataTableAdapterBatch : public detail::NoMetaInfo {
- public:
-  DataTableAdapterBatch(void** data, const char** feature_stypes,
-                        size_t num_rows, size_t num_features)
-      : data_(data),
-        feature_stypes_(feature_stypes),
-        num_features_(num_features),
-        num_rows_(num_rows) {}
+class CSCArrayAdapterBatch : public detail::NoMetaInfo {
+  ArrayInterface<1> indptr_;
+  ArrayInterface<1> indices_;
+  ArrayInterface<1> values_;
 
- private:
-  enum class DTType : uint8_t {
+  class Line {
+    std::size_t column_idx_;
+    ArrayInterface<1> row_idx_;
+    ArrayInterface<1> values_;
+    std::size_t offset_;
+
+   public:
+    Line(std::size_t idx, ArrayInterface<1> row_idx, ArrayInterface<1> values, std::size_t offset)
+        : column_idx_{idx},
+          row_idx_{std::move(row_idx)},
+          values_{std::move(values)},
+          offset_{offset} {}
+
+    std::size_t Size() const { return values_.Shape(0); }
+    COOTuple GetElement(std::size_t idx) const {
+      return {TypedIndex<std::size_t, 1>{row_idx_}(offset_ + idx), column_idx_,
+              values_(offset_ + idx)};
+    }
+  };
+
+ public:
+  static constexpr bool kIsRowMajor = false;
+
+  CSCArrayAdapterBatch(ArrayInterface<1> indptr, ArrayInterface<1> indices,
+                       ArrayInterface<1> values)
+      : indptr_{std::move(indptr)}, indices_{std::move(indices)}, values_{std::move(values)} {}
+
+  std::size_t Size() const { return indptr_.n - 1; }
+  Line GetLine(std::size_t idx) const {
+    auto begin_no_stride = TypedIndex<std::size_t, 1>{indptr_}(idx);
+    auto end_no_stride = TypedIndex<std::size_t, 1>{indptr_}(idx + 1);
+
+    auto indices = indices_;
+    auto values = values_;
+    // Slice indices and values, stride remains unchanged since this is slicing by
+    // specific index.
+    auto offset = indices.strides[0] * begin_no_stride;
+    indices.shape[0] = end_no_stride - begin_no_stride;
+    values.shape[0] = end_no_stride - begin_no_stride;
+
+    return Line{idx, indices, values, offset};
+  }
+};
+
+/**
+ * \brief CSC adapter with support for array interface.
+ */
+class CSCArrayAdapter : public detail::SingleBatchDataIter<CSCArrayAdapterBatch> {
+  ArrayInterface<1> indptr_;
+  ArrayInterface<1> indices_;
+  ArrayInterface<1> values_;
+  size_t num_rows_;
+  CSCArrayAdapterBatch batch_;
+
+ public:
+  CSCArrayAdapter(StringView indptr, StringView indices, StringView values, std::size_t num_rows)
+      : indptr_{indptr},
+        indices_{indices},
+        values_{values},
+        num_rows_{num_rows},
+        batch_{CSCArrayAdapterBatch{indptr_, indices_, values_}} {}
+
+  // JVM package sends 0 as unknown
+  size_t NumRows() const { return num_rows_ == 0 ? kAdapterUnknownSize : num_rows_; }
+  size_t NumColumns() const { return indptr_.n - 1; }
+  const CSCArrayAdapterBatch& Value() const override { return batch_; }
+};
+
+class DataTableAdapterBatch : public detail::NoMetaInfo {
+  enum class DTType : std::uint8_t {
     kFloat32 = 0,
     kFloat64 = 1,
     kBool8 = 2,
@@ -493,7 +558,7 @@ class DataTableAdapterBatch : public detail::NoMetaInfo {
     kUnknown = 7
   };
 
-  DTType DTGetType(std::string type_string) const {
+  static DTType DTGetType(std::string type_string) {
     if (type_string == "float32") {
       return DTType::kFloat32;
     } else if (type_string == "float64") {
@@ -514,8 +579,23 @@ class DataTableAdapterBatch : public detail::NoMetaInfo {
     }
   }
 
+ public:
+  DataTableAdapterBatch(void const* const* const data, char const* const* feature_stypes,
+                        std::size_t num_rows, std::size_t num_features)
+      : data_(data), num_rows_(num_rows) {
+    CHECK(feature_types_.empty());
+    std::transform(feature_stypes, feature_stypes + num_features,
+                   std::back_inserter(feature_types_),
+                   [](char const* stype) { return DTGetType(stype); });
+  }
+
+ private:
   class Line {
-    float DTGetValue(const void* column, DTType dt_type, size_t ridx) const {
+    std::size_t row_idx_;
+    void const* const* const data_;
+    std::vector<DTType> const& feature_types_;
+
+    float DTGetValue(void const* column, DTType dt_type, std::size_t ridx) const {
       float missing = std::numeric_limits<float>::quiet_NaN();
       switch (dt_type) {
         case DTType::kFloat32: {
@@ -544,8 +624,7 @@ class DataTableAdapterBatch : public detail::NoMetaInfo {
         }
         case DTType::kInt64: {
           int64_t val = reinterpret_cast<const int64_t*>(column)[ridx];
-          return val != -9223372036854775807 - 1 ? static_cast<float>(val)
-                                                 : missing;
+          return val != -9223372036854775807 - 1 ? static_cast<float>(val) : missing;
         }
         default: {
           LOG(FATAL) << "Unknown data table type.";
@@ -555,51 +634,41 @@ class DataTableAdapterBatch : public detail::NoMetaInfo {
     }
 
    public:
-    Line(DTType type, size_t size, size_t column_idx, const void* column)
-        : type_(type), size_(size), column_idx_(column_idx), column_(column) {}
-
-    size_t Size() const { return size_; }
-    COOTuple GetElement(size_t idx) const {
-      return COOTuple{idx, column_idx_, DTGetValue(column_, type_, idx)};
+    Line(std::size_t ridx, void const* const* const data, std::vector<DTType> const& ft)
+        : row_idx_{ridx}, data_{data}, feature_types_{ft} {}
+    std::size_t Size() const { return feature_types_.size(); }
+    COOTuple GetElement(std::size_t idx) const {
+      return COOTuple{row_idx_, idx, DTGetValue(data_[idx], feature_types_[idx], row_idx_)};
     }
-
-   private:
-    DTType type_;
-    size_t size_;
-    size_t column_idx_;
-    const void* column_;
   };
 
  public:
-  size_t Size() const { return num_features_; }
-  const Line GetLine(size_t idx) const {
-    return Line(DTGetType(feature_stypes_[idx]), num_rows_, idx, data_[idx]);
-  }
-  static constexpr bool kIsRowMajor = false;
+  size_t Size() const { return num_rows_; }
+  const Line GetLine(std::size_t ridx) const { return {ridx, data_, feature_types_}; }
+  static constexpr bool kIsRowMajor = true;
 
  private:
-  void** data_;
-  const char** feature_stypes_;
-  size_t num_features_;
-  size_t num_rows_;
+  void const* const* const data_;
+
+  std::vector<DTType> feature_types_;
+  std::size_t num_rows_;
 };
 
-class DataTableAdapter
-    : public detail::SingleBatchDataIter<DataTableAdapterBatch> {
+class DataTableAdapter : public detail::SingleBatchDataIter<DataTableAdapterBatch> {
  public:
-  DataTableAdapter(void** data, const char** feature_stypes, size_t num_rows,
-                   size_t num_features)
+  DataTableAdapter(void** data, const char** feature_stypes, std::size_t num_rows,
+                   std::size_t num_features)
       : batch_(data, feature_stypes, num_rows, num_features),
         num_rows_(num_rows),
         num_columns_(num_features) {}
   const DataTableAdapterBatch& Value() const override { return batch_; }
-  size_t NumRows() const { return num_rows_; }
-  size_t NumColumns() const { return num_columns_; }
+  std::size_t NumRows() const { return num_rows_; }
+  std::size_t NumColumns() const { return num_columns_; }
 
  private:
   DataTableAdapterBatch batch_;
-  size_t num_rows_;
-  size_t num_columns_;
+  std::size_t num_rows_;
+  std::size_t num_columns_;
 };
 
 class FileAdapterBatch {
@@ -1078,10 +1147,8 @@ class ArrowColumnarBatch {
 using ArrowColumnarBatchVec = std::vector<std::unique_ptr<ArrowColumnarBatch>>;
 class RecordBatchesIterAdapter: public dmlc::DataIter<ArrowColumnarBatchVec> {
  public:
-  RecordBatchesIterAdapter(XGDMatrixCallbackNext *next_callback,
-                          int nthread)
-    : next_callback_{next_callback},
-      nbatches_{nthread} {}
+  RecordBatchesIterAdapter(XGDMatrixCallbackNext* next_callback, int nbatch)
+      : next_callback_{next_callback}, nbatches_{nbatch} {}
 
   void BeforeFirst() override {
     CHECK(at_first_) << "Cannot reset RecordBatchesIterAdapter";
@@ -1131,6 +1198,23 @@ class RecordBatchesIterAdapter: public dmlc::DataIter<ArrowColumnarBatchVec> {
   struct ArrowSchemaImporter schema_;
   ArrowColumnarBatchVec batches_;
 };
+
+class SparsePageAdapterBatch {
+  HostSparsePageView page_;
+
+ public:
+  struct Line {
+    Entry const* inst;
+    size_t n;
+    bst_row_t ridx;
+    COOTuple GetElement(size_t idx) const { return {ridx, inst[idx].index, inst[idx].fvalue}; }
+    size_t Size() const { return n; }
+  };
+
+  explicit SparsePageAdapterBatch(HostSparsePageView page) : page_{std::move(page)} {}
+  Line GetLine(size_t ridx) const { return Line{page_[ridx].data(), page_[ridx].size(), ridx}; }
+  size_t Size() const { return page_.Size(); }
+};
 };  // namespace data
 }  // namespace xgboost
 #endif  // XGBOOST_DATA_ADAPTER_H_
