@@ -1,18 +1,19 @@
-/*!
- * Copyright 2014-2019 by Contributors
+/**
+ * Copyright 2014-2023 by XBGoost Contributors
  * \file updater_sync.cc
  * \brief synchronize the tree in all distributed nodes
  */
 #include <xgboost/tree_updater.h>
-#include <vector>
-#include <string>
+
 #include <limits>
+#include <string>
+#include <vector>
 
-#include "xgboost/json.h"
+#include "../collective/communicator-inl.h"
 #include "../common/io.h"
+#include "xgboost/json.h"
 
-namespace xgboost {
-namespace tree {
+namespace xgboost::tree {
 
 DMLC_REGISTRY_FILE_TAG(updater_sync);
 
@@ -20,31 +21,30 @@ DMLC_REGISTRY_FILE_TAG(updater_sync);
  * \brief syncher that synchronize the tree in all distributed nodes
  * can implement various strategies, so far it is always set to node 0's tree
  */
-class TreeSyncher: public TreeUpdater {
+class TreeSyncher : public TreeUpdater {
  public:
+  explicit TreeSyncher(Context const* tparam) : TreeUpdater(tparam) {}
   void Configure(const Args&) override {}
 
   void LoadConfig(Json const&) override {}
   void SaveConfig(Json*) const override {}
 
-  char const* Name() const override {
-    return "prune";
-  }
+  [[nodiscard]] char const* Name() const override { return "prune"; }
 
-  void Update(HostDeviceVector<GradientPair>* ,
-              DMatrix*,
-              const std::vector<RegTree*> &trees) override {
-    if (rabit::GetWorldSize() == 1) return;
+  void Update(TrainParam const*, HostDeviceVector<GradientPair>*, DMatrix*,
+              common::Span<HostDeviceVector<bst_node_t>> /*out_position*/,
+              const std::vector<RegTree*>& trees) override {
+    if (collective::GetWorldSize() == 1) return;
     std::string s_model;
     common::MemoryBufferStream fs(&s_model);
-    int rank = rabit::GetRank();
+    int rank = collective::GetRank();
     if (rank == 0) {
       for (auto tree : trees) {
         tree->Save(&fs);
       }
     }
     fs.Seek(0);
-    rabit::Broadcast(&s_model, 0);
+    collective::Broadcast(&s_model, 0);
     for (auto tree : trees) {
       tree->Load(&fs);
     }
@@ -52,9 +52,6 @@ class TreeSyncher: public TreeUpdater {
 };
 
 XGBOOST_REGISTER_TREE_UPDATER(TreeSyncher, "sync")
-.describe("Syncher that synchronize the tree in all distributed nodes.")
-.set_body([]() {
-    return new TreeSyncher();
-  });
-}  // namespace tree
-}  // namespace xgboost
+    .describe("Syncher that synchronize the tree in all distributed nodes.")
+    .set_body([](Context const* ctx, auto) { return new TreeSyncher(ctx); });
+}  // namespace xgboost::tree
