@@ -7,6 +7,13 @@ then
   echo "Activating Conda environment ${conda_env}"
   shift 1
   cmake_args="$@"
+
+  # Workaround for file permission error
+  if [[ -n $CI_BUILD_UID ]]
+  then
+    gosu root chown -R "${CI_BUILD_UID}:${CI_BUILD_GID}" /opt/mambaforge/envs
+  fi
+
   source activate ${conda_env}
   cmake_prefix_flag="-DCMAKE_PREFIX_PATH=$CONDA_PREFIX"
 else
@@ -17,7 +24,7 @@ fi
 rm -rf build
 mkdir build
 cd build
-cmake .. ${cmake_args} -DGOOGLE_TEST=ON -DUSE_DMLC_GTEST=ON -DCMAKE_VERBOSE_MAKEFILE=ON -DENABLE_ALL_WARNINGS=ON -GNinja ${cmake_prefix_flag} -DHIDE_CXX_SYMBOLS=ON
+cmake .. ${cmake_args} -DGOOGLE_TEST=ON -DUSE_DMLC_GTEST=ON -DCMAKE_VERBOSE_MAKEFILE=ON -DENABLE_ALL_WARNINGS=ON -DCMAKE_COMPILE_WARNING_AS_ERROR=ON -GNinja ${cmake_prefix_flag} -DHIDE_CXX_SYMBOLS=ON
 ninja clean
 time ninja -v
 cd ..
