@@ -28,7 +28,7 @@ function install_xgboost {
   then
     pushd .
     cd python-package
-    python setup.py install --user
+    pip install --user -v .
     popd
   fi
 }
@@ -68,18 +68,22 @@ case "$suite" in
     install_xgboost
     setup_pyspark_envs
     pytest -v -s -rxXs --fulltrace --durations=0 -m "mgpu" ${args} tests/python-gpu
+    pytest -v -s -rxXs --fulltrace --durations=0 -m "mgpu" ${args} tests/test_distributed/test_gpu_with_dask
+    pytest -v -s -rxXs --fulltrace --durations=0 -m "mgpu" ${args} tests/test_distributed/test_gpu_with_spark
     unset_pyspark_envs
     uninstall_xgboost
     set +x
     ;;
 
   cpu)
-    source activate cpu_test
+    source activate linux_cpu_test
     set -x
     install_xgboost
     export RAY_OBJECT_STORE_ALLOW_SLOW_STORAGE=1
     setup_pyspark_envs
     pytest -v -s -rxXs --fulltrace --durations=0 ${args} tests/python
+    pytest -v -s -rxXs --fulltrace --durations=0 ${args} tests/test_distributed/test_with_dask
+    pytest -v -s -rxXs --fulltrace --durations=0 ${args} tests/test_distributed/test_with_spark
     unset_pyspark_envs
     uninstall_xgboost
     set +x
