@@ -1,5 +1,5 @@
-/*!
- * Copyright (c) by Contributors 2019-2022
+/**
+ * Copyright (c) 2019-2023, XGBoost Contributors
  */
 #include <gtest/gtest.h>
 
@@ -8,8 +8,10 @@
 
 #include "../../../src/common/charconv.h"
 #include "../../../src/common/io.h"
-#include "../filesystem.h"  // dmlc::TemporaryDirectory
+#include "../../../src/common/threading_utils.h"  // for ParallelFor
+#include "../filesystem.h"                        // dmlc::TemporaryDirectory
 #include "../helpers.h"
+#include "dmlc/logging.h"
 #include "xgboost/json.h"
 #include "xgboost/json_io.h"
 #include "xgboost/logging.h"
@@ -27,7 +29,6 @@ std::string GetModelStr() {
   "train_parameter": {
     "debug_verbose": "0",
     "disable_default_eval_metric": "0",
-    "dsplit": "auto",
     "nthread": "0",
     "seed": "0",
     "seed_per_iteration": "0",
@@ -40,7 +41,6 @@ std::string GetModelStr() {
     "num_class": "0",
     "num_feature": "10",
     "objective": "reg:linear",
-    "predictor": "gpu_predictor",
     "tree_method": "gpu_hist",
     "updater": "grow_gpu_hist"
   },
@@ -590,7 +590,7 @@ TEST(Json, DISABLED_RoundTripExhaustive) {
     }
   };
   int64_t int32_max = static_cast<int64_t>(std::numeric_limits<uint32_t>::max());
-  GenericParameter ctx;
+  Context ctx;
   common::ParallelFor(int32_max, ctx.Threads(), [&](auto i) { test(static_cast<uint32_t>(i)); });
 }
 
@@ -676,4 +676,19 @@ TEST(UBJson, Basic) {
     ASSERT_FLOAT_EQ(2.71, get<Number>(get<Array>(ret["test"])[0]));
   }
 }
+
+TEST(Json, TypeCheck) {
+  Json config{Object{}};
+  config["foo"] = String{"bar"};
+  auto test = [&]() { TypeCheck<Number, Integer, Array, I32Array>(config["foo"], "foo"); };
+  ASSERT_THROW({ test(); }, dmlc::Error);
+  try {
+    test();
+  } catch (dmlc::Error const& e) {
+    auto err = std::string{e.what()};
+    ASSERT_NE(err.find("Number"), std::string::npos);
+    ASSERT_NE(err.find("I32Array"), std::string::npos);
+    ASSERT_NE(err.find("foo"), std::string::npos);
+  }
+}
 }  // namespace xgboost
