@@ -1,17 +1,15 @@
 // Copyright by Contributors
 #include <xgboost/objective.h>
-#include <xgboost/generic_parameters.h>
+#include <xgboost/context.h>
 #include <limits>
 
 #include "../helpers.h"
-
+namespace xgboost {
 TEST(Objective, DeclareUnifiedTest(HingeObj)) {
-  xgboost::GenericParameter tparam = xgboost::CreateEmptyGenericParam(GPUIDX);
-  std::unique_ptr<xgboost::ObjFunction> obj {
-    xgboost::ObjFunction::Create("binary:hinge", &tparam)
-  };
+  Context ctx = MakeCUDACtx(GPUIDX);
+  std::unique_ptr<ObjFunction> obj{ObjFunction::Create("binary:hinge", &ctx)};
 
-  xgboost::bst_float eps = std::numeric_limits<xgboost::bst_float>::min();
+  float eps = std::numeric_limits<xgboost::bst_float>::min();
   CheckObjFunction(obj,
                    {-1.0f, -0.5f, 0.5f, 1.0f, -1.0f, -0.5f,  0.5f, 1.0f},
                    { 0.0f,  0.0f, 0.0f, 0.0f,  1.0f,  1.0f,  1.0f, 1.0f},
@@ -27,3 +25,4 @@ TEST(Objective, DeclareUnifiedTest(HingeObj)) {
 
   ASSERT_NO_THROW(obj->DefaultEvalMetric());
 }
+}  // namespace xgboost
