@@ -1,21 +1,47 @@
-/*!
- * Copyright 2017-2022 by XGBoost contributors
+/**
+ * Copyright (c) 2017-2023, XGBoost contributors
  */
 #include <gtest/gtest.h>
-#include <xgboost/learner.h>
-#include <xgboost/objective.h>  // ObjFunction
-#include <xgboost/version_config.h>
+#include <xgboost/learner.h>                        // for Learner
+#include <xgboost/logging.h>                        // for LogCheck_NE, CHECK_NE, LogCheck_EQ
+#include <xgboost/objective.h>                      // for ObjFunction
+#include <xgboost/version_config.h>                 // for XGBOOST_VER_MAJOR, XGBOOST_VER_MINOR
 
-#include <string>  // std::stof, std::string
-#include <thread>
-#include <vector>
+#include <algorithm>                                // for equal, transform
+#include <cinttypes>                                // for int32_t, int64_t, uint32_t
+#include <cstddef>                                  // for size_t
+#include <iosfwd>                                   // for ofstream
+#include <iterator>                                 // for back_insert_iterator, back_inserter
+#include <limits>                                   // for numeric_limits
+#include <map>                                      // for map
+#include <memory>                                   // for unique_ptr, shared_ptr, __shared_ptr_...
+#include <random>                                   // for uniform_real_distribution
+#include <string>                                   // for allocator, basic_string, string, oper...
+#include <thread>                                   // for thread
+#include <type_traits>                              // for is_integral
+#include <utility>                                  // for pair
+#include <vector>                                   // for vector
 
-#include "../../src/common/io.h"
-#include "../../src/common/linalg_op.h"
-#include "../../src/common/random.h"
-#include "filesystem.h"  // dmlc::TemporaryDirectory
-#include "helpers.h"
-#include "xgboost/json.h"
+#include "../../src/collective/communicator-inl.h"  // for GetRank, GetWorldSize
+#include "../../src/common/api_entry.h"             // for XGBAPIThreadLocalEntry
+#include "../../src/common/io.h"                    // for LoadSequentialFile
+#include "../../src/common/linalg_op.h"             // for ElementWiseTransformHost, begin, end
+#include "../../src/common/random.h"                // for GlobalRandom
+#include "dmlc/io.h"                                // for Stream
+#include "dmlc/omp.h"                               // for omp_get_max_threads
+#include "dmlc/registry.h"                          // for Registry
+#include "filesystem.h"                             // for TemporaryDirectory
+#include "helpers.h"                                // for GetBaseScore, RandomDataGenerator
+#include "objective_helpers.h"                      // for MakeObjNamesForTest, ObjTestNameGenerator
+#include "xgboost/base.h"                           // for bst_float, Args, bst_feature_t, bst_int
+#include "xgboost/context.h"                        // for Context, DeviceOrd
+#include "xgboost/data.h"                           // for DMatrix, MetaInfo, DataType
+#include "xgboost/host_device_vector.h"             // for HostDeviceVector
+#include "xgboost/json.h"                           // for Json, Object, get, String, IsA, opera...
+#include "xgboost/linalg.h"                         // for Tensor, TensorView
+#include "xgboost/logging.h"                        // for ConsoleLogger
+#include "xgboost/predictor.h"                      // for PredictionCacheEntry
+#include "xgboost/string_view.h"                    // for StringView
 
 namespace xgboost {
 TEST(Learner, Basic) {
@@ -30,9 +56,9 @@ TEST(Learner, Basic) {
   auto minor = XGBOOST_VER_MINOR;
   auto patch = XGBOOST_VER_PATCH;
 
-  static_assert(std::is_integral<decltype(major)>::value, "Wrong major version type");
-  static_assert(std::is_integral<decltype(minor)>::value, "Wrong minor version type");
-  static_assert(std::is_integral<decltype(patch)>::value, "Wrong patch version type");
+  static_assert(std::is_integral_v<decltype(major)>, "Wrong major version type");
+  static_assert(std::is_integral_v<decltype(minor)>, "Wrong minor version type");
+  static_assert(std::is_integral_v<decltype(patch)>, "Wrong patch version type");
 }
 
 TEST(Learner, ParameterValidation) {
@@ -64,10 +90,9 @@ TEST(Learner, CheckGroup) {
   size_t constexpr kNumRows = 17;
   bst_feature_t constexpr kNumCols = 15;
 
-  std::shared_ptr<DMatrix> p_mat{
-      RandomDataGenerator{kNumRows, kNumCols, 0.0f}.GenerateDMatrix()};
-  std::vector<bst_float> weight(kNumGroups);
-  std::vector<bst_int> group(kNumGroups);
+  std::shared_ptr<DMatrix> p_mat{RandomDataGenerator{kNumRows, kNumCols, 0.0f}.GenerateDMatrix()};
+  std::vector<bst_float> weight(kNumGroups, 1);
+  std::vector<bst_group_t> group(kNumGroups);
   group[0] = 2;
   group[1] = 3;
   group[2] = 7;
@@ -98,8 +123,8 @@ TEST(Learner, SLOW_CheckMultiBatch) {  // NOLINT
   dmlc::TemporaryDirectory tempdir;
   const std::string tmp_file = tempdir.path + "/big.libsvm";
   CreateBigTestData(tmp_file, 50000);
-  std::shared_ptr<DMatrix> dmat(xgboost::DMatrix::Load(
-      tmp_file + "#" + tmp_file + ".cache", true, false, "auto"));
+  std::shared_ptr<DMatrix> dmat(
+      xgboost::DMatrix::Load(tmp_file + "?format=libsvm" + "#" + tmp_file + ".cache"));
   EXPECT_FALSE(dmat->SingleColBlock());
   size_t num_row = dmat->Info().num_row_;
   std::vector<bst_float> labels(num_row);
@@ -190,6 +215,34 @@ TEST(Learner, JsonModelIO) {
   }
 }
 
+TEST(Learner, ConfigIO) {
+  bst_row_t n_samples = 128;
+  bst_feature_t n_features = 12;
+  std::shared_ptr<DMatrix> p_fmat{
+      RandomDataGenerator{n_samples, n_features, 0}.GenerateDMatrix(true, false, 2)};
+
+  auto serialised_model_tmp = std::string{};
+  std::string eval_res_0;
+  std::string eval_res_1;
+  {
+    std::unique_ptr<Learner> learner{Learner::Create({p_fmat})};
+    learner->SetParams(Args{{"eval_metric", "ndcg"}, {"eval_metric", "map"}});
+    learner->Configure();
+    learner->UpdateOneIter(0, p_fmat);
+    eval_res_0 = learner->EvalOneIter(0, {p_fmat}, {"Train"});
+    common::MemoryBufferStream fo(&serialised_model_tmp);
+    learner->Save(&fo);
+  }
+
+  {
+    common::MemoryBufferStream fi(&serialised_model_tmp);
+    std::unique_ptr<Learner> learner{Learner::Create({p_fmat})};
+    learner->Load(&fi);
+    eval_res_1 = learner->EvalOneIter(0, {p_fmat}, {"Train"});
+  }
+  ASSERT_EQ(eval_res_0, eval_res_1);
+}
+
 // Crashes the test runner if there are race condiditions.
 //
 // Build with additional cmake flags to enable thread sanitizer
@@ -284,45 +337,36 @@ TEST(Learner, GPUConfiguration) {
     learner->SetParams({Arg{"booster", "gblinear"},
                         Arg{"updater", "gpu_coord_descent"}});
     learner->UpdateOneIter(0, p_dmat);
-    ASSERT_EQ(learner->Ctx()->gpu_id, 0);
+    ASSERT_EQ(learner->Ctx()->Device(), DeviceOrd::CUDA(0));
   }
   {
-    std::unique_ptr<Learner> learner {Learner::Create(mat)};
+    std::unique_ptr<Learner> learner{Learner::Create(mat)};
     learner->SetParams({Arg{"tree_method", "gpu_hist"}});
+    learner->Configure();
+    ASSERT_EQ(learner->Ctx()->Device(), DeviceOrd::CUDA(0));
     learner->UpdateOneIter(0, p_dmat);
-    ASSERT_EQ(learner->Ctx()->gpu_id, 0);
+    ASSERT_EQ(learner->Ctx()->Device(), DeviceOrd::CUDA(0));
   }
   {
     std::unique_ptr<Learner> learner {Learner::Create(mat)};
     learner->SetParams({Arg{"tree_method", "gpu_hist"},
                         Arg{"gpu_id", "-1"}});
     learner->UpdateOneIter(0, p_dmat);
-    ASSERT_EQ(learner->Ctx()->gpu_id, 0);
+    ASSERT_EQ(learner->Ctx()->Device(), DeviceOrd::CUDA(0));
   }
   {
     // with CPU algorithm
     std::unique_ptr<Learner> learner {Learner::Create(mat)};
     learner->SetParams({Arg{"tree_method", "hist"}});
     learner->UpdateOneIter(0, p_dmat);
-    ASSERT_EQ(learner->Ctx()->gpu_id, -1);
+    ASSERT_EQ(learner->Ctx()->Device(), DeviceOrd::CPU());
   }
   {
     // with CPU algorithm, but `gpu_id` takes priority
     std::unique_ptr<Learner> learner {Learner::Create(mat)};
-    learner->SetParams({Arg{"tree_method", "hist"},
-                        Arg{"gpu_id", "0"}});
+    learner->SetParams({Arg{"tree_method", "hist"}, Arg{"gpu_id", "0"}});
     learner->UpdateOneIter(0, p_dmat);
-    ASSERT_EQ(learner->Ctx()->gpu_id, 0);
-  }
-  {
-    // With CPU algorithm but GPU Predictor, this is to simulate when
-    // XGBoost is only used for prediction, so tree method is not
-    // specified.
-    std::unique_ptr<Learner> learner {Learner::Create(mat)};
-    learner->SetParams({Arg{"tree_method", "hist"},
-                        Arg{"predictor", "gpu_predictor"}});
-    learner->UpdateOneIter(0, p_dmat);
-    ASSERT_EQ(learner->Ctx()->gpu_id, 0);
+    ASSERT_EQ(learner->Ctx()->Device(), DeviceOrd::CUDA(0));
   }
 }
 #endif  // defined(XGBOOST_USE_CUDA)
@@ -351,6 +395,8 @@ TEST(Learner, Seed) {
 TEST(Learner, ConstantSeed) {
   auto m = RandomDataGenerator{10, 10, 0}.GenerateDMatrix(true);
   std::unique_ptr<Learner> learner{Learner::Create({m})};
+  // Use exact as it doesn't initialize column sampler at construction, which alters the rng.
+  learner->SetParam("tree_method", "exact");
   learner->Configure();  // seed the global random
 
   std::uniform_real_distribution<float> dist;
@@ -363,7 +409,7 @@ TEST(Learner, ConstantSeed) {
   CHECK_NE(v_0, v_1);
 
   {
-    rng.seed(GenericParameter::kDefaultSeed);
+    rng.seed(Context::kDefaultSeed);
     std::uniform_real_distribution<float> dist;
     float v_2 = dist(rng);
     CHECK_EQ(v_0, v_2);
@@ -460,10 +506,6 @@ class InitBaseScore : public ::testing::Test {
 
   void SetUp() override { Xy_ = RandomDataGenerator{10, Cols(), 0}.GenerateDMatrix(true); }
 
-  static float GetBaseScore(Json const &config) {
-    return std::stof(get<String const>(config["learner"]["learner_model_param"]["base_score"]));
-  }
-
  public:
   void TestUpdateConfig() {
     std::unique_ptr<Learner> learner{Learner::Create({Xy_})};
@@ -611,4 +653,69 @@ TEST_F(InitBaseScore, InitAfterLoad) { this->TestInitAfterLoad(); }
 TEST_F(InitBaseScore, InitWithPredict) { this->TestInitWithPredt(); }
 
 TEST_F(InitBaseScore, UpdateProcess) { this->TestUpdateProcess(); }
+
+class TestColumnSplit : public ::testing::TestWithParam<std::string> {
+  void TestBaseScore(std::string objective, float expected_base_score, Json expected_model) {
+    auto const world_size = collective::GetWorldSize();
+    auto const rank = collective::GetRank();
+
+    auto p_fmat = MakeFmatForObjTest(objective);
+    std::shared_ptr<DMatrix> sliced{p_fmat->SliceCol(world_size, rank)};
+    std::unique_ptr<Learner> learner{Learner::Create({sliced})};
+    learner->SetParam("tree_method", "approx");
+    learner->SetParam("objective", objective);
+    if (objective.find("quantile") != std::string::npos) {
+      learner->SetParam("quantile_alpha", "0.5");
+    }
+    if (objective.find("multi") != std::string::npos) {
+      learner->SetParam("num_class", "3");
+    }
+    learner->UpdateOneIter(0, sliced);
+    Json config{Object{}};
+    learner->SaveConfig(&config);
+    auto base_score = GetBaseScore(config);
+    ASSERT_EQ(base_score, expected_base_score);
+
+    Json model{Object{}};
+    learner->SaveModel(&model);
+    ASSERT_EQ(model, expected_model);
+  }
+
+ public:
+  void Run(std::string objective) {
+    auto p_fmat = MakeFmatForObjTest(objective);
+    std::unique_ptr<Learner> learner{Learner::Create({p_fmat})};
+    learner->SetParam("tree_method", "approx");
+    learner->SetParam("objective", objective);
+    if (objective.find("quantile") != std::string::npos) {
+      learner->SetParam("quantile_alpha", "0.5");
+    }
+    if (objective.find("multi") != std::string::npos) {
+      learner->SetParam("num_class", "3");
+    }
+    learner->UpdateOneIter(0, p_fmat);
+
+    Json config{Object{}};
+    learner->SaveConfig(&config);
+
+    Json model{Object{}};
+    learner->SaveModel(&model);
+
+    auto constexpr kWorldSize{3};
+    auto call = [this, &objective](auto&... args) { TestBaseScore(objective, args...); };
+    auto score = GetBaseScore(config);
+    RunWithInMemoryCommunicator(kWorldSize, call, score, model);
+  }
+};
+
+TEST_P(TestColumnSplit, Objective) {
+  std::string objective = GetParam();
+  this->Run(objective);
+}
+
+INSTANTIATE_TEST_SUITE_P(ColumnSplitObjective, TestColumnSplit,
+                         ::testing::ValuesIn(MakeObjNamesForTest()),
+                         [](const ::testing::TestParamInfo<TestColumnSplit::ParamType>& info) {
+                           return ObjTestNameGenerator(info);
+                         });
 }  // namespace xgboost
