@@ -1,13 +1,18 @@
-import sys
 import os
+import sys
+
 import numpy as np
-import xgboost as xgb
 import pytest
+
+import xgboost as xgb
+from xgboost import testing as tm
+
 sys.path.append("tests/python")
+import test_basic_models as test_bm
+
 # Don't import the test class, otherwise they will run twice.
 import test_callback as test_cb  # noqa
-import test_basic_models as test_bm
-import testing as tm
+
 rng = np.random.RandomState(1994)
 
 
@@ -37,9 +42,15 @@ class TestGPUBasicModels:
     def test_custom_objective(self):
         self.cpu_test_bm.run_custom_objective("gpu_hist")
 
-    def test_eta_decay_gpu_hist(self):
+    def test_eta_decay(self):
         self.cpu_test_cb.run_eta_decay('gpu_hist')
 
+    @pytest.mark.parametrize(
+        "objective", ["binary:logistic", "reg:absoluteerror", "reg:quantileerror"]
+    )
+    def test_eta_decay_leaf_output(self, objective) -> None:
+        self.cpu_test_cb.run_eta_decay_leaf_output("gpu_hist", objective)
+
     def test_deterministic_gpu_hist(self):
         kRows = 1000
         kCols = 64
@@ -54,16 +65,20 @@ class TestGPUBasicModels:
     @pytest.mark.skipif(**tm.no_sklearn())
     def test_invalid_gpu_id(self):
         from sklearn.datasets import load_digits
+
         X, y = load_digits(return_X_y=True)
         # should pass with invalid gpu id
-        cls1 = xgb.XGBClassifier(tree_method='gpu_hist', gpu_id=9999)
+        cls1 = xgb.XGBClassifier(tree_method="gpu_hist", gpu_id=9999)
         cls1.fit(X, y)
         # should throw error with fail_on_invalid_gpu_id enabled
         cls2 = xgb.XGBClassifier(
-            tree_method='gpu_hist', gpu_id=9999, fail_on_invalid_gpu_id=True
+            tree_method="gpu_hist", gpu_id=9999, fail_on_invalid_gpu_id=True
         )
-        try:
+        with pytest.raises(ValueError, match="ordinal 9999 is invalid"):
+            cls2.fit(X, y)
+
+        cls2 = xgb.XGBClassifier(
+            tree_method="hist", device="cuda:9999", fail_on_invalid_gpu_id=True
+        )
+        with pytest.raises(ValueError, match="ordinal 9999 is invalid"):
             cls2.fit(X, y)
-            assert False, "Should have failed with with fail_on_invalid_gpu_id enabled"
-        except xgb.core.XGBoostError as err:
-            assert "gpu_id 9999 is invalid" in str(err)
