@@ -4,11 +4,12 @@ import tempfile
 import numpy as np
 import pytest
 import scipy.sparse
-import testing as tm
 from hypothesis import given, settings, strategies
 from scipy.sparse import csr_matrix, rand
 
 import xgboost as xgb
+from xgboost import testing as tm
+from xgboost.testing.data import np_dtypes
 
 rng = np.random.RandomState(1)
 
@@ -81,10 +82,6 @@ class TestDMatrix:
 
             assert len(record) == 0
 
-        with pytest.warns(UserWarning):
-            csr = csr_matrix(x)
-            xgb.DMatrix(csr.tocsc(), y, missing=4)
-
     def test_dmatrix_numpy_init(self):
         data = np.random.randn(5, 5)
         dm = xgb.DMatrix(data)
@@ -129,6 +126,12 @@ class TestDMatrix:
         assert dtrain.num_row() == 3
         assert dtrain.num_col() == 3
 
+        indptr = np.array([0, 3, 5])
+        data = np.array([0, 1, 2, 3, 4])
+        row_idx = np.array([0, 1, 2, 0, 2])
+        X = scipy.sparse.csc_matrix((data, row_idx, indptr), shape=(3, 2))
+        assert tm.predictor_equal(xgb.DMatrix(X.tocsr()), xgb.DMatrix(X))
+
     def test_coo(self):
         row = np.array([0, 2, 2, 0, 1, 2])
         col = np.array([0, 0, 1, 2, 2, 2])
@@ -216,8 +219,8 @@ class TestDMatrix:
         assert dm.slice([0, 1]).num_col() == dm.num_col()
         assert dm.slice([0, 1]).feature_names == dm.feature_names
 
-        dm.feature_types = 'q'
-        assert dm.feature_types == list('qqqqq')
+        with pytest.raises(ValueError, match=r"Duplicates found: \['bar'\]"):
+            dm.feature_names = ["bar"] * (data.shape[1] - 2) + ["a", "b"]
 
         dm.feature_types = list('qiqiq')
         assert dm.feature_types == list('qiqiq')
@@ -227,6 +230,7 @@ class TestDMatrix:
 
         # reset
         dm.feature_names = None
+        dm.feature_types = None
         assert dm.feature_names is None
         assert dm.feature_types is None
 
@@ -280,7 +284,7 @@ class TestDMatrix:
             assert m0.feature_types == m1.feature_types
 
     def test_get_info(self):
-        dtrain = xgb.DMatrix(dpath + 'agaricus.txt.train')
+        dtrain, _ = tm.load_agaricus(__file__)
         dtrain.get_float_info('label')
         dtrain.get_float_info('weight')
         dtrain.get_float_info('base_margin')
@@ -429,7 +433,9 @@ class TestDMatrix:
     def test_uri_categorical(self):
         path = os.path.join(dpath, 'agaricus.txt.train')
         feature_types = ["q"] * 5 + ["c"] + ["q"] * 120
-        Xy = xgb.DMatrix(path + "?indexing_mode=1", feature_types=feature_types)
+        Xy = xgb.DMatrix(
+            path + "?indexing_mode=1&format=libsvm", feature_types=feature_types
+        )
         np.testing.assert_equal(np.array(Xy.feature_types), np.array(feature_types))
 
     def test_base_margin(self):
@@ -453,3 +459,11 @@ class TestDMatrix:
         np.testing.assert_equal(csr.indptr, ret.indptr)
         np.testing.assert_equal(csr.data, ret.data)
         np.testing.assert_equal(csr.indices, ret.indices)
+
+    def test_dtypes(self) -> None:
+        n_samples = 128
+        n_features = 16
+        for orig, x in np_dtypes(n_samples, n_features):
+            m0 = xgb.DMatrix(orig)
+            m1 = xgb.DMatrix(x)
+            assert tm.predictor_equal(m0, m1)
