@@ -1,11 +1,13 @@
-import os
-import tempfile
+from typing import Type
+
 import numpy as np
-import xgboost as xgb
-import testing as tm
 import pytest
 from test_dmatrix import set_base_margin_info
 
+import xgboost as xgb
+from xgboost import testing as tm
+from xgboost.testing.data import pd_arrow_dtypes, pd_dtypes
+
 try:
     import pandas as pd
 except ImportError:
@@ -209,7 +211,7 @@ class TestPandas:
         y = np.random.randn(kRows)
         w = np.random.uniform(size=kRows).astype(np.float32)
         w_pd = pd.DataFrame(w)
-        data = xgb.DMatrix(X, y, w_pd)
+        data = xgb.DMatrix(X, y, weight=w_pd)
 
         assert data.num_row() == kRows
         assert data.num_col() == kCols
@@ -220,7 +222,7 @@ class TestPandas:
         set_base_margin_info(pd.DataFrame, xgb.DMatrix, "hist")
 
     def test_cv_as_pandas(self):
-        dm = xgb.DMatrix(dpath + 'agaricus.txt.train')
+        dm, _ = tm.load_agaricus(__file__)
         params = {'max_depth': 2, 'eta': 1, 'verbosity': 0,
                   'objective': 'binary:logistic', 'eval_metric': 'error'}
 
@@ -297,70 +299,62 @@ class TestPandas:
         assert 'auc' not in cv.columns[0]
         assert 'error' in cv.columns[0]
 
-    def test_nullable_type(self):
-        y = np.random.default_rng(0).random(4)
+    @pytest.mark.parametrize("DMatrixT", [xgb.DMatrix, xgb.QuantileDMatrix])
+    def test_nullable_type(self, DMatrixT) -> None:
+        from xgboost.data import is_pd_cat_dtype
 
-        def to_bytes(Xy: xgb.DMatrix) -> bytes:
-            with tempfile.TemporaryDirectory() as tmpdir:
-                path = os.path.join(tmpdir, "Xy.dmatrix")
-                Xy.save_binary(path)
-                with open(path, "rb") as fd:
-                    result = fd.read()
-            return result
+        for orig, df in pd_dtypes():
+            if hasattr(df.dtypes, "__iter__"):
+                enable_categorical = any(is_pd_cat_dtype(dtype) for dtype in df.dtypes)
+            else:
+                # series
+                enable_categorical = is_pd_cat_dtype(df.dtype)
 
-        def test_int(dtype) -> bytes:
-            arr = pd.DataFrame(
-                {"f0": [1, 2, None, 3], "f1": [4, 3, None, 1]}, dtype=dtype
-            )
-            Xy = xgb.DMatrix(arr, y)
-            Xy.feature_types = None
-            return to_bytes(Xy)
+            f0_orig = orig[orig.columns[0]] if isinstance(orig, pd.DataFrame) else orig
+            f0 = df[df.columns[0]] if isinstance(df, pd.DataFrame) else df
+            y_orig = f0_orig.astype(pd.Float32Dtype()).fillna(0)
+            y = f0.astype(pd.Float32Dtype()).fillna(0)
 
-        b0 = test_int(np.float32)
-        b1 = test_int(pd.Int16Dtype())
-        assert b0 == b1
+            m_orig = DMatrixT(orig, enable_categorical=enable_categorical, label=y_orig)
+            # extension types
+            copy = df.copy()
+            m_etype = DMatrixT(df, enable_categorical=enable_categorical, label=y)
+            # no mutation
+            assert df.equals(copy)
+            # different from pd.BooleanDtype(), None is converted to False with bool
+            if hasattr(orig.dtypes, "__iter__") and any(
+                dtype == "bool" for dtype in orig.dtypes
+            ):
+                assert not tm.predictor_equal(m_orig, m_etype)
+            else:
+                assert tm.predictor_equal(m_orig, m_etype)
 
-        def test_bool(dtype) -> bytes:
-            arr = pd.DataFrame(
-                {"f0": [True, False, None, True], "f1": [False, True, None, True]},
-                dtype=dtype,
-            )
-            Xy = xgb.DMatrix(arr, y)
-            Xy.feature_types = None
-            return to_bytes(Xy)
+            np.testing.assert_allclose(m_orig.get_label(), m_etype.get_label())
+            np.testing.assert_allclose(m_etype.get_label(), y.values.astype(np.float32))
 
-        b0 = test_bool(pd.BooleanDtype())
-        b1 = test_bool(bool)
-        assert b0 != b1         # None is converted to False with np.bool
+            if isinstance(df, pd.DataFrame):
+                f0 = df["f0"]
+                with pytest.raises(ValueError, match="Label contains NaN"):
+                    xgb.DMatrix(df, f0, enable_categorical=enable_categorical)
 
-        data = {"f0": [1.0, 2.0, None, 3.0], "f1": [3.0, 2.0, None, 1.0]}
+    @pytest.mark.skipif(**tm.no_arrow())
+    @pytest.mark.parametrize("DMatrixT", [xgb.DMatrix, xgb.QuantileDMatrix])
+    def test_pyarrow_type(self, DMatrixT: Type[xgb.DMatrix]) -> None:
+        for orig, df in pd_arrow_dtypes():
+            f0_orig: pd.Series = orig["f0"]
+            f0 = df["f0"]
 
-        arr = np.array([data["f0"], data["f1"]]).T
-        Xy = xgb.DMatrix(arr, y)
-        Xy.feature_types = None
-        Xy.feature_names = None
-        from_np = to_bytes(Xy)
+            if f0.dtype.name.startswith("bool"):
+                y = None
+                y_orig = None
+            else:
+                y_orig = f0_orig.fillna(0, inplace=False)
+                y = f0.fillna(0, inplace=False)
 
-        def test_float(dtype) -> bytes:
-            arr = pd.DataFrame(data, dtype=dtype)
-            Xy = xgb.DMatrix(arr, y)
-            Xy.feature_types = None
-            Xy.feature_names = None
-            return to_bytes(Xy)
+            m_orig = DMatrixT(orig, enable_categorical=True, label=y_orig)
+            m_etype = DMatrixT(df, enable_categorical=True, label=y)
 
-        b0 = test_float(pd.Float64Dtype())
-        b1 = test_float(float)
-        assert b0 == b1         # both are converted to NaN
-        assert b0 == from_np
-
-        def test_cat(dtype) -> bytes:
-            arr = pd.DataFrame(data, dtype=dtype)
-            if dtype is None:
-                arr = arr.astype("category")
-            Xy = xgb.DMatrix(arr, y, enable_categorical=True)
-            Xy.feature_types = None
-            return to_bytes(Xy)
-
-        b0 = test_cat(pd.CategoricalDtype())
-        b1 = test_cat(None)
-        assert b0 == b1
+            assert tm.predictor_equal(m_orig, m_etype)
+            if y is not None:
+                np.testing.assert_allclose(m_orig.get_label(), m_etype.get_label())
+                np.testing.assert_allclose(m_etype.get_label(), y.values)
